@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -16,15 +16,60 @@ import org.eclipse.swt.widgets.*;
 
 class PromptDialog extends Dialog {
 	
-	public PromptDialog(Shell parent, int style) {
+	PromptDialog(Shell parent, int style) {
 		super(parent, style);
 	}
 	
-	public PromptDialog(Shell parent) {
+	PromptDialog(Shell parent) {
 		this(parent, 0);
 	}
 	
-	public void confirmEx(String title, String text, String check, String button1, String button2, String button3, final int[] checkValue, final int[] result) {
+	void alertCheck(String title, String text, String check, final int[] checkValue) {
+		Shell parent = getParent();
+		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
+		if (title != null) shell.setText(title);
+		GridLayout gridLayout = new GridLayout();
+		shell.setLayout(gridLayout);
+		Label label = new Label(shell, SWT.WRAP);
+		label.setText(text);
+		GridData data = new GridData();
+		Monitor monitor = parent.getMonitor();
+		int maxWidth = monitor.getBounds().width * 2 / 3;
+		int width = label.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
+		data.widthHint = Math.min(width, maxWidth);
+		data.horizontalAlignment = GridData.FILL;
+		data.grabExcessHorizontalSpace = true;
+		label.setLayoutData (data);
+
+		final Button checkButton = check != null ? new Button(shell, SWT.CHECK) : null;
+		if (checkButton != null) {
+			checkButton.setText(check);
+			checkButton.setSelection(checkValue[0] != 0);
+			data = new GridData ();
+			data.horizontalAlignment = GridData.BEGINNING;
+			checkButton.setLayoutData (data);
+		}
+		Button okButton = new Button(shell, SWT.PUSH);
+		okButton.setText(SWT.getMessage("SWT_OK")); //$NON-NLS-1$
+		data = new GridData ();
+		data.horizontalAlignment = GridData.CENTER;
+		okButton.setLayoutData (data);
+		okButton.addListener(SWT.Selection, new Listener() {
+			public void handleEvent(Event event) {
+				if (checkButton != null) checkValue[0] = checkButton.getSelection() ? 1 : 0;
+				shell.close();
+			}
+		});
+
+		shell.pack();
+		shell.open();
+		Display display = parent.getDisplay();
+		while (!shell.isDisposed()) {
+			if (!display.readAndDispatch()) display.sleep();
+		}
+	}
+
+	void confirmEx(String title, String text, String check, String button0, String button1, String button2, int defaultIndex, final int[] checkValue, final int[] result) {
 		Shell parent = getParent();
 		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
 		shell.setText(title);
@@ -33,10 +78,14 @@ class PromptDialog extends Dialog {
 		Label label = new Label(shell, SWT.WRAP);
 		label.setText(text);
 		GridData data = new GridData();
+		Monitor monitor = parent.getMonitor();
+		int maxWidth = monitor.getBounds().width * 2 / 3;
+		int width = label.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
+		data.widthHint = Math.min(width, maxWidth);
 		data.horizontalAlignment = GridData.FILL;
 		data.grabExcessHorizontalSpace = true;
 		label.setLayoutData (data);
-		
+
 		final Button[] buttons = new Button[4];
 		Listener listener = new Listener() {
 			public void handleEvent(Event event) {
@@ -56,29 +105,42 @@ class PromptDialog extends Dialog {
 			buttons[0].setText(check);
 			buttons[0].setSelection(checkValue[0] != 0);
 			data = new GridData ();
-			data.horizontalAlignment = GridData.END;
+			data.horizontalAlignment = GridData.BEGINNING;
 			buttons[0].setLayoutData (data);
 		}
 		Composite composite = new Composite(shell, SWT.NONE);
 		data = new GridData();
-		data.horizontalAlignment = GridData.END;
+		data.horizontalAlignment = GridData.CENTER;
 		composite.setLayoutData (data);
-		composite.setLayout(new RowLayout());
-		if (button1 != null) {
+		GridLayout layout = new GridLayout();
+		layout.makeColumnsEqualWidth = true;
+		composite.setLayout(layout);
+		int buttonCount = 0;
+		if (button0 != null) {
 			buttons[1] = new Button(composite, SWT.PUSH);
-			buttons[1].setText(button1);
+			buttons[1].setText(button0);
 			buttons[1].addListener(SWT.Selection, listener);
+			buttons[1].setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+			buttonCount++;
+		}
+		if (button1 != null) {
+			buttons[2] = new Button(composite, SWT.PUSH);
+			buttons[2].setText(button1);
+			buttons[2].addListener(SWT.Selection, listener);
+			buttons[2].setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+			buttonCount++;
 		}
 		if (button2 != null) {
-			buttons[2] = new Button(composite, SWT.PUSH);
-			buttons[2].setText(button2);
-			buttons[2].addListener(SWT.Selection, listener);
-		}
-		if (button3 != null) {
 			buttons[3] = new Button(composite, SWT.PUSH);
-			buttons[3].setText(button3);
+			buttons[3].setText(button2);
 			buttons[3].addListener(SWT.Selection, listener);
+			buttons[3].setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+			buttonCount++;
 		}
+		layout.numColumns = buttonCount;
+		Button defaultButton = buttons [defaultIndex + 1];
+		if (defaultButton != null) shell.setDefaultButton (defaultButton);
+
 		shell.pack();
 		shell.open();
 		Display display = parent.getDisplay();
@@ -87,7 +149,7 @@ class PromptDialog extends Dialog {
 		}
 	}
 	
-	public void prompt(String title, String text, String check, final String[] value, final int[] checkValue, final int[] result) {
+	void prompt(String title, String text, String check, final String[] value, final int[] checkValue, final int[] result) {
 		Shell parent = getParent();
 		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
 		if (title != null) shell.setText(title);
@@ -96,6 +158,10 @@ class PromptDialog extends Dialog {
 		Label label = new Label(shell, SWT.WRAP);
 		label.setText(text);
 		GridData data = new GridData();
+		Monitor monitor = parent.getMonitor();
+		int maxWidth = monitor.getBounds().width * 2 / 3;
+		int width = label.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
+		data.widthHint = Math.min(width, maxWidth);
 		data.horizontalAlignment = GridData.FILL;
 		data.grabExcessHorizontalSpace = true;
 		label.setLayoutData (data);
@@ -103,10 +169,12 @@ class PromptDialog extends Dialog {
 		final Text valueText = new Text(shell, SWT.BORDER);
 		if (value[0] != null) valueText.setText(value[0]);
 		data = new GridData();
+		width = valueText.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
+		if (width > maxWidth) data.widthHint = maxWidth;
 		data.horizontalAlignment = GridData.FILL;
 		data.grabExcessHorizontalSpace = true;
 		valueText.setLayoutData(data);
-				
+
 		final Button[] buttons = new Button[3];
 		Listener listener = new Listener() {
 			public void handleEvent(Event event) {
@@ -121,12 +189,12 @@ class PromptDialog extends Dialog {
 			buttons[0].setText(check);
 			buttons[0].setSelection(checkValue[0] != 0);
 			data = new GridData ();
-			data.horizontalAlignment = GridData.END;
+			data.horizontalAlignment = GridData.BEGINNING;
 			buttons[0].setLayoutData (data);
 		}
 		Composite composite = new Composite(shell, SWT.NONE);
 		data = new GridData();
-		data.horizontalAlignment = GridData.END;
+		data.horizontalAlignment = GridData.CENTER;
 		composite.setLayoutData (data);
 		composite.setLayout(new GridLayout(2, true));
 		buttons[1] = new Button(composite, SWT.PUSH);
@@ -145,7 +213,8 @@ class PromptDialog extends Dialog {
 			if (!display.readAndDispatch()) display.sleep();
 		}	
 	}
-	public void promptUsernameAndPassword(String title, String text, String check, final String[] user, final String[] pass, final int[] checkValue, final int[] result) {
+
+	void promptUsernameAndPassword(String title, String text, String check, final String[] user, final String[] pass, final int[] checkValue, final int[] result) {
 		Shell parent = getParent();
 		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
 		shell.setText(title);
@@ -154,6 +223,10 @@ class PromptDialog extends Dialog {
 		Label label = new Label(shell, SWT.WRAP);
 		label.setText(text);
 		GridData data = new GridData();
+		Monitor monitor = parent.getMonitor();
+		int maxWidth = monitor.getBounds().width * 2 / 3;
+		int width = label.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
+		data.widthHint = Math.min(width, maxWidth);
 		data.horizontalAlignment = GridData.FILL;
 		data.grabExcessHorizontalSpace = true;
 		label.setLayoutData (data);
@@ -193,11 +266,12 @@ class PromptDialog extends Dialog {
 			buttons[0].setText(check);
 			buttons[0].setSelection(checkValue[0] != 0);
 			data = new GridData ();
+			data.horizontalAlignment = GridData.BEGINNING;
 			buttons[0].setLayoutData (data);
 		}
 		Composite composite = new Composite(shell, SWT.NONE);
 		data = new GridData();
-		data.horizontalAlignment = GridData.END;
+		data.horizontalAlignment = GridData.CENTER;
 		composite.setLayoutData (data);
 		composite.setLayout(new GridLayout(2, true));
 		buttons[1] = new Button(composite, SWT.PUSH);
