@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -90,6 +90,7 @@ import org.eclipse.swt.graphics.*;
  * @see #getMaximized
  * @see Shell
  * @see SWT
+ * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public class Decorations extends Canvas {
 	String text;
@@ -100,7 +101,7 @@ public class Decorations extends Canvas {
 	Menu [] menus;
 	Control savedFocus;
 	Button defaultButton, saveDefault;
-	int /*long*/ accelGroup;	
+	int /*long*/ accelGroup, vboxHandle;
 	
 Decorations () {
 	/* Do nothing */
@@ -188,7 +189,7 @@ void _setImages (Image [] images) {
 	if (pixbufs != 0) OS.g_list_free (pixbufs);
 }
 
-void add (Menu menu) {
+void addMenu (Menu menu) {
 	if (menus == null) menus = new Menu [4];
 	for (int i=0; i<menus.length; i++) {
 		if (menus [i] == null) {
@@ -286,7 +287,7 @@ void fixDecorations (Decorations newDecorations, Control control, Menu [] menus)
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #setDefaultButton
+ * @see #setDefaultButton(Button)
  */
 public Button getDefaultButton () {
 	checkWidget();
@@ -447,7 +448,7 @@ Decorations menuShell () {
 	return this;
 }
 
-void remove (Menu menu) {
+void removeMenu (Menu menu) {
 	if (menus == null) return;
 	for (int i=0; i<menus.length; i++) {
 		if (menus [i] == menu) {
@@ -457,9 +458,12 @@ void remove (Menu menu) {
 	}
 }
 
-void releaseWidget () {
-	if (menuBar != null) menuBar.releaseResources ();
-	menuBar = null;
+void releaseChildren (boolean destroy) {
+	if (menuBar != null) {
+		menuBar.release (false);
+		menuBar = null;
+	}
+	super.releaseChildren (destroy);
 	if (menus != null) {
 		for (int i=0; i<menus.length; i++) {
 			Menu menu = menus [i];
@@ -467,8 +471,16 @@ void releaseWidget () {
 				menu.dispose ();
 			}
 		}
+		menus = null;
 	}
-	menus = null;
+}
+
+void releaseHandle () {
+	super.releaseHandle ();
+	vboxHandle = 0;
+}
+
+void releaseWidget () {
 	super.releaseWidget ();
 	image = null;
 	images = null;
@@ -511,6 +523,7 @@ boolean restoreFocus () {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the button has been disposed</li> 
+ *    <li>ERROR_INVALID_PARENT - if the control is not in the same widget tree</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -522,6 +535,7 @@ public void setDefaultButton (Button button) {
 	int /*long*/ buttonHandle = 0;
 	if (button != null) {
 		if (button.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+		if (button.menuShell () != this) error (SWT.ERROR_INVALID_PARENT);
 		buttonHandle = button.handle;
 	}
 	saveDefault = defaultButton = button;
