@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -28,12 +28,15 @@ import org.eclipse.swt.graphics.*;
  * <p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
+ *
+ * @see <a href="http://www.eclipse.org/swt/snippets/#table">Table, TableItem, TableColumn snippets</a>
+ * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public class TableItem extends Item {
 	Table parent;
 	Font font;
-	boolean cached, grayed;
 	Font[] cellFont;
+	boolean cached, grayed;
 	
 /**
  * Constructs a new instance of this class given its parent
@@ -52,10 +55,11 @@ public class TableItem extends Item {
  *
  * @param parent a composite control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -121,6 +125,81 @@ static Table checkNull (Table control) {
 	return control;
 }
 
+Color _getBackground () {
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.BACKGROUND_COLUMN, ptr, -1);
+	if (ptr [0] == 0) return parent.getBackground ();
+	GdkColor gdkColor = new GdkColor ();
+	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
+	return Color.gtk_new (display, gdkColor);
+}
+
+Color _getBackground (int index) {
+	int count = Math.max (1, parent.columnCount);
+	if (0 > index || index > count - 1) return _getBackground ();
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
+	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_BACKGROUND, ptr, -1);
+	if (ptr [0] == 0) return _getBackground ();
+	GdkColor gdkColor = new GdkColor ();
+	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
+	return Color.gtk_new (display, gdkColor);
+}
+
+boolean _getChecked () {
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.CHECKED_COLUMN, ptr, -1);
+	return ptr [0] != 0;
+}
+
+Color _getForeground () {
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.FOREGROUND_COLUMN, ptr, -1);
+	if (ptr [0] == 0) return parent.getForeground ();
+	GdkColor gdkColor = new GdkColor ();
+	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
+	return Color.gtk_new (display, gdkColor);
+}
+
+Color _getForeground (int index) {
+	int count = Math.max (1, parent.columnCount);
+	if (0 > index || index > count - 1) return _getForeground ();
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	int modelIndex =  parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
+	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_FOREGROUND, ptr, -1);
+	if (ptr [0] == 0) return _getForeground ();
+	GdkColor gdkColor = new GdkColor ();
+	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
+	return Color.gtk_new (display, gdkColor);
+}
+
+Image _getImage (int index) {
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return null;
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
+	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_PIXBUF, ptr, -1);
+	if (ptr [0] == 0) return null;
+	ImageList imageList = parent.imageList;
+	int imageIndex = imageList.indexOf (ptr [0]);
+	if (imageIndex == -1) return null;
+	return imageList.get (imageIndex);
+}
+
+String _getText (int index) {
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return "";
+	int /*long*/ [] ptr = new int /*long*/ [1];
+	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
+	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_TEXT, ptr, -1);
+	if (ptr [0] == 0) return "";
+	int length = OS.strlen (ptr [0]);
+	byte[] buffer = new byte [length];
+	OS.memmove (buffer, ptr [0], length);
+	OS.g_free (ptr [0]);
+	return new String (Converter.mbcsToWcs (null, buffer));
+}
+
 protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
@@ -133,32 +212,26 @@ void clear () {
 			OS.gtk_list_store_set (parent.modelHandle, handle, i, 0, -1);
 		}
 		/*
-		* Bug in GTK.  When using fixed-height-mode on versions before 2.6.3,
+		* Bug in GTK.  When using fixed-height-mode,
 		* row changes do not cause the row to be repainted.  The fix is to
 		* invalidate the row when it is cleared.
 		*/
 		if ((parent.style & SWT.VIRTUAL) != 0) {
 			if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
-				if ((OS.GTK_WIDGET_FLAGS (parent.handle) & OS.GTK_REALIZED) != 0) {
-					int /*long*/ parentHandle = parent.handle;
-					int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
-					GdkRectangle rect = new GdkRectangle ();
-					OS.gtk_tree_view_get_cell_area (parentHandle, path, 0, rect);
-					OS.gtk_tree_path_free (path);
-					int /*long*/ window = OS.gtk_tree_view_get_bin_window (parentHandle);
-					rect.x = 0;
-					rect.width = OS.GTK_WIDGET_WIDTH (parentHandle);
-					OS.gdk_window_invalidate_rect (window, rect, false);
-				}
+				redraw ();
 			}
 		}
-		
 	}
 	cached = false;
 	font = null;
 	cellFont = null;
 }
 
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
+}
+
 /**
  * Returns the receiver's background color.
  *
@@ -174,12 +247,79 @@ void clear () {
 public Color getBackground () {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.BACKGROUND_COLUMN, ptr, -1);
-	if (ptr [0] == 0) return parent.getBackground ();
-	GdkColor gdkColor = new GdkColor ();
-	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
-	return Color.gtk_new (display, gdkColor);
+	return _getBackground ();
+}
+
+/**
+ * Returns a rectangle describing the receiver's size and location
+ * relative to its parent.
+ *
+ * @return the receiver's bounding rectangle
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public Rectangle getBounds () {
+	// TODO fully test on early and later versions of GTK
+	// shifted a bit too far right on later versions of GTK - however, old Tree also had this problem
+	checkWidget ();
+	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
+	int /*long*/ parentHandle = parent.handle;
+	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, 0);
+	if (column == 0) return new Rectangle (0, 0, 0, 0);
+	int /*long*/ textRenderer = parent.getTextRenderer (column);
+	int /*long*/ pixbufRenderer = parent.getPixbufRenderer (column);
+	if (textRenderer == 0 || pixbufRenderer == 0)  return new Rectangle (0, 0, 0, 0);
+
+	int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
+	OS.gtk_widget_realize (parentHandle);
+	
+	boolean isExpander = OS.gtk_tree_model_iter_n_children (parent.modelHandle, handle) > 0;
+	boolean isExpanded = OS.gtk_tree_view_row_expanded (parentHandle, path);
+	OS.gtk_tree_view_column_cell_set_cell_data (column, parent.modelHandle, handle, isExpander, isExpanded);
+	
+	GdkRectangle rect = new GdkRectangle ();
+	OS.gtk_tree_view_get_cell_area (parentHandle, path, column, rect);
+	OS.gtk_tree_path_free (path);
+	if ((parent.getStyle () & SWT.MIRRORED) != 0) rect.x = parent.getClientWidth () - rect.width - rect.x;
+	int right = rect.x + rect.width;
+
+	int [] x = new int [1], w = new int [1];
+	parent.ignoreSize = true;
+	OS.gtk_cell_renderer_get_size (textRenderer, parentHandle, null, null, null, w, null);
+	parent.ignoreSize = false;
+	rect.width = w [0];
+	int [] buffer = new int [1];
+	if (OS.gtk_tree_view_get_expander_column (parentHandle) == column) {
+		OS.gtk_widget_style_get (parentHandle, OS.expander_size, buffer, 0);
+		rect.x += buffer [0] + TreeItem.EXPANDER_EXTRA_PADDING;
+	}
+	OS.gtk_widget_style_get (parentHandle, OS.horizontal_separator, buffer, 0);
+	int horizontalSeparator = buffer[0];
+	rect.x += horizontalSeparator;
+	
+	if (OS.GTK_VERSION >= OS.VERSION (2, 1, 3)) {
+		OS.gtk_tree_view_column_cell_get_position (column, textRenderer, x, null);
+		rect.x += x [0];
+	} else {
+		if ((parent.style & SWT.CHECK) != 0) {
+			OS.gtk_cell_renderer_get_size (parent.checkRenderer, parentHandle, null, null, null, w, null);
+			rect.x += w [0] + horizontalSeparator;
+		}
+		OS.gtk_cell_renderer_get_size (pixbufRenderer, parentHandle, null, null, null, w, null);
+		rect.x += w [0] + horizontalSeparator;
+	}
+	if (parent.columnCount > 0) {
+		if (rect.x + rect.width > right) {
+			rect.width = Math.max (0, right - rect.x);
+		}
+	}
+	int width = OS.gtk_tree_view_column_get_visible (column) ? rect.width + 1 : 0;
+	return new Rectangle (rect.x, rect.y, width, rect.height + 1);
 }
 
 /**
@@ -198,15 +338,7 @@ public Color getBackground () {
 public Color getBackground (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int count = Math.max (1, parent.columnCount);
-	if (0 > index || index > count - 1) return getBackground ();
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
-	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_BACKGROUND, ptr, -1);
-	if (ptr [0] == 0) return getBackground ();
-	GdkColor gdkColor = new GdkColor ();
-	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
-	return Color.gtk_new (display, gdkColor);
+	return _getBackground (index);
 }
 
 /**
@@ -237,6 +369,7 @@ public Rectangle getBounds (int index) {
 	GdkRectangle rect = new GdkRectangle ();
 	OS.gtk_tree_view_get_cell_area (parentHandle, path, column, rect);
 	OS.gtk_tree_path_free (path);
+	if ((parent.getStyle () & SWT.MIRRORED) != 0) rect.x = parent.getClientWidth () - rect.width - rect.x;
 	
 	if (index == 0 && (parent.style & SWT.CHECK) != 0) {
 		if (OS.GTK_VERSION >= OS.VERSION (2, 1, 3)) {
@@ -253,23 +386,8 @@ public Rectangle getBounds (int index) {
 			rect.width -= w [0]  + buffer [0];
 		}
 	}
-	
-	/* 
-	 * In the horizontal direction, the origin of the bin window is 
-	 * not the same as the origin of the scrolled handle.
-	 * The method gtk_tree_view_get_cell_area returns the 
-	 * x coordinate relative to the bin window.  In order to
-	 * get the coordinates relative to the top left corner
-	 * of the client area,  we need to account for the
-	 * horizontal scroll adjustment.
-	 */
-	int[] wx = new int[1];
-	OS.gtk_tree_view_tree_to_widget_coords(parentHandle, rect.x, 0, wx, null);
-	rect.x = wx[0];
-
-	int border = parent.getBorderWidth ();
-	int headerHeight = parent.getHeaderHeight ();
-	return new Rectangle (rect.x + border, rect.y + (headerHeight == 0 ?  border : headerHeight), rect.width + 1, rect.height + 1);
+	int width = OS.gtk_tree_view_column_get_visible (column) ? rect.width + 1 : 0;
+	return new Rectangle (rect.x, rect.y, width, rect.height + 1);
 }
 
 /**
@@ -288,9 +406,7 @@ public boolean getChecked () {
 	checkWidget();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	if ((parent.style & SWT.CHECK) == 0) return false;
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.CHECKED_COLUMN, ptr, -1);
-	return ptr[0] != 0;
+	return _getChecked ();
 }
 
 /**
@@ -349,12 +465,7 @@ public Font getFont (int index) {
 public Color getForeground () {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	OS.gtk_tree_model_get (parent.modelHandle, handle, Table.FOREGROUND_COLUMN, ptr, -1);
-	if (ptr [0] == 0) return parent.getForeground ();
-	GdkColor gdkColor = new GdkColor ();
-	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
-	return Color.gtk_new (display, gdkColor);
+	return _getForeground ();
 }
 
 /**
@@ -374,15 +485,7 @@ public Color getForeground () {
 public Color getForeground (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int count = Math.max (1, parent.columnCount);
-	if (0 > index || index > count - 1) return getForeground ();
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	int modelIndex =  parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
-	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_FOREGROUND, ptr, -1);
-	if (ptr [0] == 0) return getForeground ();
-	GdkColor gdkColor = new GdkColor ();
-	OS.memmove (gdkColor, ptr [0], GdkColor.sizeof);
-	return Color.gtk_new (display, gdkColor);
+	return _getForeground (index);
 }
 
 /**
@@ -425,16 +528,7 @@ public Image getImage () {
 public Image getImage (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int count = Math.max (1, parent.getColumnCount ());
-	if (0 > index || index > count - 1) return null;
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
-	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_PIXBUF, ptr, -1);
-	if (ptr [0] == 0) return null;
-	ImageList imageList = parent.imageList;
-	int imageIndex = imageList.indexOf (ptr [0]);
-	if (imageIndex == -1) return null;
-	return imageList.get (imageIndex);
+	return _getImage (index);
 }
 
 /**
@@ -469,7 +563,7 @@ public Rectangle getImageBounds (int index) {
 	OS.gtk_widget_realize (parentHandle);
 	OS.gtk_tree_view_get_cell_area (parentHandle, path, column, rect);
 	OS.gtk_tree_path_free (path);
-
+	if ((parent.getStyle () & SWT.MIRRORED) != 0) rect.x = parent.getClientWidth () - rect.width - rect.x;
 	/*
 	* The OS call gtk_cell_renderer_get_size() provides the width of image to be drawn
 	* by the cell renderer.  If there is no image in the cell, the width is zero.  If the table contains
@@ -490,23 +584,8 @@ public Rectangle getImageBounds (int index) {
 		OS.gtk_cell_renderer_get_size (pixbufRenderer, parentHandle, null, null, null, w, null);
 		rect.width = w [0];
 	}
-	
-	/* 
-	 * In the horizontal direction, the origin of the bin window is 
-	 * not the same as the origin of the scrolled handle.
-	 * The method gtk_tree_view_get_cell_area returns the 
-	 * x coordinate relative to the bin window.  In order to
-	 * get the coordinates relative to the top left corner
-	 * of the client area,  we need to account for the
-	 * horizontal scroll adjustment.
-	 */
-	int[] wx = new int[1];
-	OS.gtk_tree_view_tree_to_widget_coords(parentHandle, rect.x, 0, wx, null);
-	rect.x = wx[0];
-	
-	int border = parent.getBorderWidth ();
-	int headerHeight = parent.getHeaderHeight ();
-	return new Rectangle (rect.x + border, rect.y + (headerHeight == 0 ?  border : headerHeight), rect.width, rect.height + 1);
+	int width = OS.gtk_tree_view_column_get_visible (column) ? rect.width : 0;
+	return new Rectangle (rect.x, rect.y, width, rect.height + 1);
 }
 
 /**
@@ -569,29 +648,115 @@ public String getText () {
 public String getText (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	int count = Math.max (1, parent.getColumnCount ());
-	if (0 > index || index > count - 1) return "";
-	int /*long*/ [] ptr = new int /*long*/ [1];
-	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
-	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Table.CELL_TEXT, ptr, -1);
-	if (ptr [0] == 0) return "";
-	int length = OS.strlen (ptr [0]);
-	byte[] buffer = new byte [length];
-	OS.memmove (buffer, ptr [0], length);
-	OS.g_free (ptr [0]);
-	return new String (Converter.mbcsToWcs (null, buffer));
+	return _getText (index);
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
+/**
+ * Returns a rectangle describing the size and location
+ * relative to its parent of the text at a column in the
+ * table.  An empty rectangle is returned if index exceeds
+ * the index of the table's last column.
+ *
+ * @param index the index that specifies the column
+ * @return the receiver's bounding text rectangle
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.3
+ */
+public Rectangle getTextBounds (int index) {
+	checkWidget ();
+	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return new Rectangle (0, 0, 0, 0);
+	// TODO fully test on early and later versions of GTK
+	// shifted a bit too far right on later versions of GTK - however, old Tree also had this problem
+	int /*long*/ parentHandle = parent.handle;
+	int /*long*/ column = 0;
+	if (index >= 0 && index < parent.columnCount) {
+		column = parent.columns [index].handle;
+	} else {
+		column = OS.gtk_tree_view_get_column (parentHandle, index);
+	}
+	if (column == 0) return new Rectangle (0, 0, 0, 0);
+	int /*long*/ textRenderer = parent.getTextRenderer (column);
+	int /*long*/ pixbufRenderer = parent.getPixbufRenderer (column);
+	if (textRenderer == 0 || pixbufRenderer == 0)  return new Rectangle (0, 0, 0, 0);
+
+	int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
+	OS.gtk_widget_realize (parentHandle);
+	
+	boolean isExpander = OS.gtk_tree_model_iter_n_children (parent.modelHandle, handle) > 0;
+	boolean isExpanded = OS.gtk_tree_view_row_expanded (parentHandle, path);
+	OS.gtk_tree_view_column_cell_set_cell_data (column, parent.modelHandle, handle, isExpander, isExpanded);
+	
+	GdkRectangle rect = new GdkRectangle ();
+	OS.gtk_tree_view_get_cell_area (parentHandle, path, column, rect);
+	OS.gtk_tree_path_free (path);
+	if ((parent.getStyle () & SWT.MIRRORED) != 0) rect.x = parent.getClientWidth () - rect.width - rect.x;
+	int right = rect.x + rect.width;
+
+	int [] x = new int [1], w = new int [1];
+	parent.ignoreSize = true;
+	OS.gtk_cell_renderer_get_size (textRenderer, parentHandle, null, null, null, w, null);
+	parent.ignoreSize = false;
+	int [] buffer = new int [1];
+	if (OS.gtk_tree_view_get_expander_column (parentHandle) == column) {
+		OS.gtk_widget_style_get (parentHandle, OS.expander_size, buffer, 0);
+		rect.x += buffer [0] + TreeItem.EXPANDER_EXTRA_PADDING;
+	}
+	OS.gtk_widget_style_get (parentHandle, OS.horizontal_separator, buffer, 0);
+	int horizontalSeparator = buffer[0];
+	rect.x += horizontalSeparator;
+	
+	if (OS.GTK_VERSION >= OS.VERSION (2, 1, 3)) {
+		OS.gtk_tree_view_column_cell_get_position (column, textRenderer, x, null);
+		rect.x += x [0];
+	} else {
+		if ((parent.style & SWT.CHECK) != 0) {
+			OS.gtk_cell_renderer_get_size (parent.checkRenderer, parentHandle, null, null, null, w, null);
+			rect.x += w [0] + horizontalSeparator;
+		}
+		OS.gtk_cell_renderer_get_size (pixbufRenderer, parentHandle, null, null, null, w, null);
+		rect.x += w [0] + horizontalSeparator;
+	}
+	if (parent.columnCount > 0) {
+		if (rect.x + rect.width > right) {
+			rect.width = Math.max (0, right - rect.x);
+		}
+	}
+	int width = OS.gtk_tree_view_column_get_visible (column) ? rect.width + 1 : 0;
+	return new Rectangle (rect.x, rect.y, width, rect.height + 1);
+}
+
+void redraw () {
+	if ((OS.GTK_WIDGET_FLAGS (parent.handle) & OS.GTK_REALIZED) != 0) {
+		int /*long*/ parentHandle = parent.handle;
+		int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
+		GdkRectangle rect = new GdkRectangle ();
+		OS.gtk_tree_view_get_cell_area (parentHandle, path, 0, rect);
+		OS.gtk_tree_path_free (path);
+		int /*long*/ window = OS.gtk_tree_view_get_bin_window (parentHandle);
+		rect.x = 0;
+		int [] w = new int [1], h = new int [1];
+		OS.gdk_drawable_get_size (window, w, h);
+		rect.width = w [0];
+		OS.gdk_window_invalidate_rect (window, rect, false);
+	}
+}
+
+void releaseHandle () {
+	if (handle != 0) OS.g_free (handle);
+	handle = 0;
+	super.releaseHandle ();
+	parent = null;
 }
 
 void releaseWidget () {
 	super.releaseWidget ();
-	if (handle != 0) OS.g_free (handle);
-	handle = 0;
-	parent = null;
 	font = null;
 	cellFont = null;
 }
@@ -618,8 +783,19 @@ public void setBackground (Color color) {
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	if (_getBackground ().equals (color)) return;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_list_store_set (parent.modelHandle, handle, Table.BACKGROUND_COLUMN, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -646,11 +822,22 @@ public void setBackground (int index, Color color) {
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	if (_getBackground (index).equals (color)) return;
 	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_list_store_set (parent.modelHandle, handle, modelIndex + Table.CELL_BACKGROUND, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 	
 	if (color != null) {
@@ -667,8 +854,8 @@ public void setBackground (int index, Color color) {
 				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parentHandle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -693,6 +880,7 @@ public void setBackground (int index, Color color) {
 public void setChecked (boolean checked) {
 	checkWidget();
 	if ((parent.style & SWT.CHECK) == 0) return;
+	if (_getChecked () == checked) return;
 	OS.gtk_list_store_set (parent.modelHandle, handle, Table.CHECKED_COLUMN, checked, -1);
 	/*
 	* GTK+'s "inconsistent" state does not match SWT's concept of grayed.  To
@@ -725,11 +913,22 @@ public void setFont (Font font){
 	if (font != null && font.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (this.font == font) return;
-	if (this.font != null && this.font.equals (font)) return;
+	Font oldFont = this.font;
+	if (oldFont == font) return;
 	this.font = font;
+	if (oldFont != null && oldFont.equals (font)) return;
 	int /*long*/ fontHandle = font != null ? font.handle : 0;
 	OS.gtk_list_store_set (parent.modelHandle, handle, Table.FONT_COLUMN, fontHandle, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -760,15 +959,27 @@ public void setFont (int index, Font font) {
 	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	if (cellFont == null) {
+		if (font == null) return;
 		cellFont = new Font [count];
 	}
-	if (cellFont [index] == font) return;
-	if (cellFont [index] != null && cellFont [index].equals (font)) return;
+	Font oldFont = cellFont [index];
+	if (oldFont == font) return;
 	cellFont [index] = font;
+	if (oldFont != null && oldFont.equals (font)) return;
 
 	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
 	int /*long*/ fontHandle  = font != null ? font.handle : 0;
 	OS.gtk_list_store_set (parent.modelHandle, handle, modelIndex + Table.CELL_FONT, fontHandle, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;	
 	
 	if (font != null) {
@@ -785,8 +996,8 @@ public void setFont (int index, Font font) {
 				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parentHandle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -819,8 +1030,19 @@ public void setForeground (Color color){
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	if (_getForeground ().equals (color)) return;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_list_store_set (parent.modelHandle, handle, Table.FOREGROUND_COLUMN, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -847,11 +1069,22 @@ public void setForeground (int index, Color color){
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	if (_getForeground (index).equals (color)) return;
 	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_list_store_set (parent.modelHandle, handle, modelIndex + Table.CELL_FOREGROUND, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 	
 	if (color != null) {
@@ -865,10 +1098,11 @@ public void setForeground (int index, Color color){
 				} else {
 					column = OS.gtk_tree_view_get_column (parentHandle, index);
 				}
+				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parentHandle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -893,6 +1127,7 @@ public void setForeground (int index, Color color){
 public void setGrayed (boolean grayed) {
 	checkWidget();
 	if ((parent.style & SWT.CHECK) == 0) return;
+	if (this.grayed == grayed) return;
 	this.grayed = grayed;
 	/*
 	* GTK+'s "inconsistent" state does not match SWT's concept of grayed.
@@ -920,9 +1155,12 @@ public void setGrayed (boolean grayed) {
  */
 public void setImage (int index, Image image) {
 	checkWidget ();
-	if (image != null && image.isDisposed()) {
+	if (image != null && image.isDisposed ()) {
 		error(SWT.ERROR_INVALID_ARGUMENT);
 	}
+	if (image != null && image.type == SWT.ICON) {
+		if (image.equals (_getImage (index))) return;
+	}
 	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	int /*long*/ pixbuf = 0;
@@ -935,6 +1173,41 @@ public void setImage (int index, Image image) {
 	}
 	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_list_store_set (parent.modelHandle, handle, modelIndex + Table.CELL_PIXBUF, pixbuf, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
+	/*
+	 * Bug in GTK.  When in fixed height mode, GTK does not recalculate the cell renderer width
+	 * when the image is changed in the model.  The fix is to force it to recalculate the width if
+	 * more space is required.
+	 */
+	if ((parent.style & SWT.VIRTUAL) != 0 && parent.currentItem == null) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2)) {
+			if (image != null) {
+				int /*long*/parentHandle = parent.handle;
+				int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
+				int [] w = new int [1];
+				int /*long*/ pixbufRenderer = parent.getPixbufRenderer(column);
+				OS.gtk_tree_view_column_cell_get_position (column, pixbufRenderer, null, w);
+				if (w[0] < image.getBounds().width) {
+					/*
+					* There is no direct way to clear the cell renderer width so we
+					* are relying on the fact that it is done as part of modifying
+					* the style.
+					*/
+					int /*long*/ style = OS.gtk_widget_get_modifier_style (parentHandle);
+					parent.modifyStyle (parentHandle, style);
+				}
+			} 
+		}
+	}
 	cached = true;
 }
 
@@ -1002,11 +1275,22 @@ public void setImageIndent (int indent) {
 public void setText (int index, String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (_getText (index).equals (string)) return;
 	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	byte[] buffer = Converter.wcsToMbcs (null, string, true);
 	int modelIndex = parent.columnCount == 0 ? Table.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_list_store_set (parent.modelHandle, handle, modelIndex + Table.CELL_TEXT, buffer, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
