@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,24 +20,38 @@ import org.eclipse.swt.events.*;
 /**
  * Instances of this class are selectable user interface
  * objects that allow the user to enter and modify text.
+ * Text controls can be either single or multi-line.
+ * When a text control is created with a border, the
+ * operating system includes a platform specific inset
+ * around the contents of the control.  When created
+ * without a border, an effort is made to remove the
+ * inset such that the preferred size of the control
+ * is the same size as the contents.
  * <p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>CENTER, LEFT, MULTI, PASSWORD, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
+ * <dd>CANCEL, CENTER, LEFT, MULTI, PASSWORD, SEARCH, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
  * <dt><b>Events:</b></dt>
  * <dd>DefaultSelection, Modify, Verify</dd>
  * </dl>
  * <p>
- * Note: Only one of the styles MULTI and SINGLE may be specified. 
+ * Note: Only one of the styles MULTI and SINGLE may be specified,
+ * and only one of the styles LEFT, CENTER, and RIGHT may be specified.
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
+ *
+ * @see <a href="http://www.eclipse.org/swt/snippets/#text">Text snippets</a>
+ * @see <a href="http://www.eclipse.org/swt/examples.php">SWT Example: ControlExample</a>
+ * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public class Text extends Scrollable {
 	int /*long*/ bufferHandle;
 	int tabs = 8, lastEventTime = 0;
 	int /*long*/ gdkEventKey = 0;
+	int fixStart = -1, fixEnd = -1;
 	boolean doubleClick;
+	String message = "";
 	
 	static final int INNER_BORDER = 2;
 	static final int ITER_SIZEOF = OS.GtkTextIter_sizeof();
@@ -103,6 +117,11 @@ public Text (Composite parent, int style) {
 }
 
 static int checkStyle (int style) {
+	if ((style & SWT.SEARCH) != 0) {
+		style |= SWT.SINGLE | SWT.BORDER;
+		style &= ~SWT.PASSWORD;
+	}
+	style &= ~SWT.SEARCH;
 	if ((style & SWT.SINGLE) != 0 && (style & SWT.MULTI) != 0) {
 		style &= ~SWT.MULTI;
 	}
@@ -146,7 +165,7 @@ void createHandle (int index) {
 		OS.gtk_container_add (fixedHandle, scrolledHandle);
 		OS.gtk_container_add (scrolledHandle, handle);
 		OS.gtk_text_view_set_editable (handle, (style & SWT.READ_ONLY) == 0);
-		if ((style & SWT.WRAP) != 0) OS.gtk_text_view_set_wrap_mode (handle, OS.GTK_WRAP_WORD);
+		if ((style & SWT.WRAP) != 0) OS.gtk_text_view_set_wrap_mode (handle, OS.GTK_WRAP_WORD_CHAR);
 		int hsp = (style & SWT.H_SCROLL) != 0 ? OS.GTK_POLICY_ALWAYS : OS.GTK_POLICY_NEVER;
 		int vsp = (style & SWT.V_SCROLL) != 0 ? OS.GTK_POLICY_ALWAYS : OS.GTK_POLICY_NEVER;
 		OS.gtk_scrolled_window_set_policy (scrolledHandle, hsp, vsp);
@@ -193,15 +212,17 @@ public void addModifyListener (ModifyListener listener) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the control is selected by the user, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
  * <code>widgetSelected</code> is not called for texts.
- * <code>widgetDefaultSelected</code> is typically called when ENTER is pressed in a single-line text.
+ * <code>widgetDefaultSelected</code> is typically called when ENTER is pressed in a single-line text,
+ * or when ENTER is pressed in a search text. If the receiver has the <code>SWT.SEARCH | SWT.CANCEL</code> style
+ * and the user cancels the search, the event object detail field contains the value <code>SWT.CANCEL</code>.
  * </p>
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should be notified when the control is selected by the user
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -312,6 +333,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	if (hHint != SWT.DEFAULT && hHint < 0) hHint = 0;
 	int[] w = new int [1], h = new int [1];
 	if ((style & SWT.SINGLE) != 0) {
+		OS.gtk_widget_realize (handle);
 		int /*long*/ layout = OS.gtk_entry_get_layout (handle);
 		OS.pango_layout_get_size (layout, w, h);
 	} else {
@@ -326,6 +348,15 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	}
 	int width = OS.PANGO_PIXELS (w [0]);
 	int height = OS.PANGO_PIXELS (h [0]);
+	//This code is intentionally commented
+//	if ((style & SWT.SEARCH) != 0 && message.length () != 0) {
+//		GC gc = new GC (this);
+//		Point size = gc.stringExtent (message);
+//		width = Math.max (width, size.x);
+//		gc.dispose ();
+//	}
+	if (width == 0) width = DEFAULT_WIDTH;
+	if (height == 0) height = DEFAULT_HEIGHT;
 	width = wHint == SWT.DEFAULT ? width : wHint;
 	height = hHint == SWT.DEFAULT ? height : hHint;
 	Rectangle trim = computeTrim (0, 0, width, height);
@@ -413,6 +444,55 @@ void deregister () {
 	if (imContext != 0) display.removeWidget (imContext);
 }
 
+boolean dragDetect (int x, int y, boolean filter, boolean [] consume) {
+	if (filter) {
+		int start = 0, end = 0;
+		if ((style & SWT.SINGLE) != 0) {
+			int [] s = new int [1], e = new int [1];
+			OS.gtk_editable_get_selection_bounds (handle, s, e);
+			start = s [0];
+			end = e [0];
+		} else {
+			byte [] s = new byte [ITER_SIZEOF], e =  new byte [ITER_SIZEOF];
+			OS.gtk_text_buffer_get_selection_bounds (bufferHandle, s, e);
+			start = OS.gtk_text_iter_get_offset (s);
+			end = OS.gtk_text_iter_get_offset (e);
+		}
+		if (start != end) {
+			if (end < start) {
+				int temp = end;
+				end = start;
+				start = temp;
+			}
+			int position = -1;
+			if ((style & SWT.SINGLE) != 0) {
+				int [] index = new int [1];
+				int [] trailing = new int [1];
+				int /*long*/ layout = OS.gtk_entry_get_layout (handle);
+				OS.pango_layout_xy_to_index (layout, x * OS.PANGO_SCALE, y * OS.PANGO_SCALE, index, trailing);
+				int /*long*/ ptr = OS.pango_layout_get_text (layout);
+				position = (int)/*64*/OS.g_utf8_pointer_to_offset (ptr, ptr + index[0]) + trailing[0];
+			} else {
+				byte [] p = new byte [ITER_SIZEOF];
+				OS.gtk_text_view_get_iter_at_location (handle, p, x, y);
+				position = OS.gtk_text_iter_get_offset (p);
+			}
+			if (start <= position && position < end) {
+				if (super.dragDetect (x, y, filter, consume)) {
+					if (consume != null) consume [0] = true;
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+	return super.dragDetect (x, y, filter, consume);
+}
+
+int /*long*/ eventWindow () {
+	return paintWindow ();
+}
+
 boolean filterKey (int keyval, int /*long*/ event) {
 	int time = OS.gdk_event_get_time (event);
 	if (time != lastEventTime) {
@@ -498,7 +578,20 @@ public int getCaretLineNumber () {
  */
 public Point getCaretLocation () {
 	checkWidget ();
-	if ((style & SWT.SINGLE) != 0) return new Point (0, 0);
+	if ((style & SWT.SINGLE) != 0) {
+		int index = OS.gtk_editable_get_position (handle);
+		if (OS.GTK_VERSION >= OS.VERSION (2, 6, 0)) {
+			index = OS.gtk_entry_text_index_to_layout_index (handle, index);
+		}
+		int [] offset_x = new int [1], offset_y = new int [1];
+		OS.gtk_entry_get_layout_offsets (handle, offset_x, offset_y);
+		int /*long*/ layout = OS.gtk_entry_get_layout (handle);
+		PangoRectangle pos = new PangoRectangle ();
+		OS.pango_layout_index_to_pos (layout, index, pos);
+		int x = offset_x [0] + OS.PANGO_PIXELS (pos.x) - getBorderWidth ();
+		int y = offset_y [0] + OS.PANGO_PIXELS (pos.y);
+		return new Point (x, y);
+	}
 	byte [] position = new byte [ITER_SIZEOF];
 	int /*long*/ mark = OS.gtk_text_buffer_get_insert (bufferHandle);
 	OS.gtk_text_buffer_get_iter_at_mark (bufferHandle, position, mark);
@@ -603,7 +696,7 @@ public char getEchoChar () {
 /**
  * Returns the editable state.
  *
- * @return whether or not the reciever is editable
+ * @return whether or not the receiver is editable
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -670,6 +763,30 @@ public int getLineHeight () {
 	return fontHeight (getFontDescription (), handle);
 }
 
+/**
+ * Returns the widget message. When the widget is created
+ * with the style <code>SWT.SEARCH</code>, the message text
+ * is displayed as a hint for the user, indicating the
+ * purpose of the field.
+ * <p>
+ * Note: This operation is a <em>HINT</em> and is not
+ * supported on platforms that do not have this concept.
+ * </p>
+ * 
+ * @return the widget message
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.3
+ */
+public String getMessage () {
+	checkWidget ();
+	return message;
+}
+
 /**
  * Returns the orientation of the receiver, which will be one of the
  * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
@@ -688,6 +805,25 @@ public int getOrientation () {
 	return style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT);
 }
 
+/*public*/ int getPosition (Point point) {
+	checkWidget ();
+	if (point == null) error (SWT.ERROR_NULL_ARGUMENT);
+	int position = -1;
+	if ((style & SWT.SINGLE) != 0) {
+		int [] index = new int [1];
+		int [] trailing = new int [1];
+		int /*long*/ layout = OS.gtk_entry_get_layout (handle);
+		OS.pango_layout_xy_to_index (layout, point.x * OS.PANGO_SCALE, point.y * OS.PANGO_SCALE, index, trailing);
+		int /*long*/ ptr = OS.pango_layout_get_text (layout);
+		position = (int)/*64*/OS.g_utf8_pointer_to_offset (ptr, ptr + index[0]) + trailing[0];
+	} else {
+		byte [] p = new byte [ITER_SIZEOF];
+		OS.gtk_text_view_get_iter_at_location (handle, p, point.x, point.y);
+		position = OS.gtk_text_iter_get_offset (p);
+	}
+	return position;
+}
+
 /**
  * Returns a <code>Point</code> whose x coordinate is the
  * character position representing the start of the selected
@@ -935,13 +1071,13 @@ public int getTopPixel () {
 	return lineTop [0];
 }
 
-int gtk_activate (int widget) {
+int /*long*/ gtk_activate (int /*long*/ widget) {
 	postEvent (SWT.DefaultSelection);
 	return 0;
 }
 
 int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result =  super.gtk_button_press_event (widget, event);
+	int /*long*/ result = super.gtk_button_press_event (widget, event);
 	if (result != 0) return result;
 	GdkEventButton gdkEvent = new GdkEventButton ();
 	OS.memmove (gdkEvent, event, GdkEventButton.sizeof);
@@ -955,8 +1091,32 @@ int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
 	return result;
 }
 
+
 int /*long*/ gtk_changed (int /*long*/ widget) {
-	sendEvent (SWT.Modify);
+	/*
+	* Feature in GTK.  When the user types, GTK positions
+	* the caret after sending the changed signal.  This
+	* means that application code that attempts to position
+	* the caret during a changed signal will fail.  The fix
+	* is to post the modify event when the user is typing.
+	*/
+	boolean keyPress = false;
+	int /*long*/ eventPtr = OS.gtk_get_current_event ();
+	if (eventPtr != 0) {
+		GdkEventKey gdkEvent = new GdkEventKey ();
+		OS.memmove (gdkEvent, eventPtr, GdkEventKey.sizeof);
+		switch (gdkEvent.type) {
+			case OS.GDK_KEY_PRESS:
+				keyPress = true;
+				break;
+		}
+		OS.gdk_event_free (eventPtr);
+	}
+	if (keyPress) {
+		postEvent (SWT.Modify);
+	} else {
+		sendEvent (SWT.Modify);
+	}
 	return 0;
 }
 
@@ -972,6 +1132,14 @@ int /*long*/ gtk_commit (int /*long*/ imContext, int /*long*/ text) {
 	char [] chars = Converter.mbcsToWcs (null, buffer);
 	char [] newChars = sendIMKeyEvent (SWT.KeyDown, null, chars);
 	if (newChars == null) return 0;
+	/*
+	* Feature in GTK.  For a GtkEntry, during the insert-text signal,
+	* GTK allows the programmer to change only the caret location,
+	* not the selection.  If the programmer changes the selection,
+	* the new selection is lost.  The fix is to detect a selection
+	* change and set it after the insert-text signal has completed.
+	*/
+	fixStart = fixEnd = -1;
 	OS.g_signal_handlers_block_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
 	int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
 	int mask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
@@ -984,6 +1152,13 @@ int /*long*/ gtk_commit (int /*long*/ imContext, int /*long*/ text) {
 	}
 	OS.g_signal_handlers_unblock_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
 	OS.g_signal_handlers_block_matched (imContext, mask, id, 0, 0, 0, handle);
+	if ((style & SWT.SINGLE) != 0) { 
+		if (fixStart != -1 && fixEnd != -1) {
+			OS.gtk_editable_set_position (handle, fixStart);
+			OS.gtk_editable_select_region (handle, fixStart, fixEnd);
+		}
+	}
+	fixStart = fixEnd = -1;
 	return 0;
 }
 
@@ -997,6 +1172,14 @@ int /*long*/ gtk_delete_range (int /*long*/ widget, int /*long*/ iter1, int /*lo
 	int end = OS.gtk_text_iter_get_offset (endIter);
 	String newText = verifyText ("", start, end);
 	if (newText == null) {
+		/* Remember the selection when the text was deleted */
+		OS.gtk_text_buffer_get_selection_bounds (bufferHandle, startIter, endIter);
+		start = OS.gtk_text_iter_get_offset (startIter);
+		end = OS.gtk_text_iter_get_offset (endIter);
+		if (start != end) {
+			fixStart = start;
+			fixEnd = end;
+		}
 		OS.g_signal_stop_emission_by_name (bufferHandle, OS.delete_range);
 	} else {
 		if (newText.length () > 0) {
@@ -1006,9 +1189,9 @@ int /*long*/ gtk_delete_range (int /*long*/ widget, int /*long*/ iter1, int /*lo
 			OS.gtk_text_buffer_delete (bufferHandle, startIter, endIter);
 			OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_RANGE);
 			OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
-			OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+			OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
 			OS.gtk_text_buffer_insert (bufferHandle, startIter, buffer, buffer.length);
-			OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+			OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
 			OS.g_signal_stop_emission_by_name (bufferHandle, OS.delete_range);
 		}
 	}
@@ -1019,6 +1202,13 @@ int /*long*/ gtk_delete_text (int /*long*/ widget, int /*long*/ start_pos, int /
 	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
 	String newText = verifyText ("", (int)/*64*/start_pos, (int)/*64*/end_pos);
 	if (newText == null) {
+		/* Remember the selection when the text was deleted */
+		int [] newStart = new int [1], newEnd = new int [1];
+		OS.gtk_editable_get_selection_bounds (handle, newStart, newEnd);
+		if (newStart [0] != newEnd [0]) {
+			fixStart = newStart [0];
+			fixEnd = newEnd [0];
+		}
 		OS.g_signal_stop_emission_by_name (handle, OS.delete_text);
 	} else {
 		if (newText.length () > 0) {
@@ -1038,6 +1228,26 @@ int /*long*/ gtk_delete_text (int /*long*/ widget, int /*long*/ start_pos, int /
 
 int /*long*/ gtk_event_after (int /*long*/ widget, int /*long*/ gdkEvent) {
 	if (cursor != null) setCursor (cursor.handle);
+	/*
+	* Feature in GTK.  The gtk-entry-select-on-focus property is a global
+	* setting.  Return it to its default value after the GtkEntry has done
+	* its focus in processing so that other widgets (such as the combo)
+	* use the correct value.
+	*/
+	if ((style & SWT.SINGLE) != 0 && display.entrySelectOnFocus) {
+		GdkEvent event = new GdkEvent ();
+		OS.memmove (event, gdkEvent, GdkEvent.sizeof);
+		switch (event.type) {
+			case OS.GDK_FOCUS_CHANGE:
+				GdkEventFocus gdkEventFocus = new GdkEventFocus ();
+				OS.memmove (gdkEventFocus, gdkEvent, GdkEventFocus.sizeof);
+				if (gdkEventFocus.in == 0) {
+					int /*long*/ settings = OS.gtk_settings_get_default ();
+					OS.g_object_set (settings, OS.gtk_entry_select_on_focus, true, 0);
+				}
+				break;
+		}
+	}
 	return super.gtk_event_after (widget, gdkEvent);
 }
 
@@ -1046,62 +1256,66 @@ int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
 	return super.gtk_focus_out_event (widget, event);
 }
 
+int /*long*/ gtk_grab_focus (int /*long*/ widget) {
+	int /*long*/ result = super.gtk_grab_focus (widget);
+	/*
+	* Feature in GTK.  GtkEntry widgets select their text on focus in,
+	* clearing the previous selection.  This behavior is controlled by
+	* the gtk-entry-select-on-focus property.  The fix is to disable
+	* this property when a GtkEntry is given focus and restore it after
+	* the entry has done focus in processing.
+	*/
+	if ((style & SWT.SINGLE) != 0 && display.entrySelectOnFocus) {
+		int /*long*/ settings = OS.gtk_settings_get_default ();
+		OS.g_object_set (settings, OS.gtk_entry_select_on_focus, false, 0);
+	}
+	return result;
+}
+
 int /*long*/ gtk_insert_text (int /*long*/ widget, int /*long*/ new_text, int /*long*/ new_text_length, int /*long*/ position) {
 	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
-	if ((style & SWT.SINGLE) != 0) {
-		if (new_text == 0 || new_text_length == 0) return 0;
-		byte [] buffer = new byte [(int)/*64*/new_text_length];
-		OS.memmove (buffer, new_text, buffer.length);
-		String oldText = new String (Converter.mbcsToWcs (null, buffer));
-		int [] pos = new int [1];
-		OS.memmove (pos, position, 4);
-		if (pos [0] == -1) pos [0] = getCharCount ();
-		int [] oldStart = new int [1], oldEnd = new int [1];
-		OS.gtk_editable_get_selection_bounds (handle, oldStart, oldEnd);
-		String newText = verifyText (oldText, pos [0], pos [0]);
-		final int [] newStart = new int [1], newEnd = new int [1];
+	if (new_text == 0 || new_text_length == 0) return 0;
+	byte [] buffer = new byte [(int)/*64*/new_text_length];
+	OS.memmove (buffer, new_text, buffer.length);
+	String oldText = new String (Converter.mbcsToWcs (null, buffer));
+	int [] pos = new int [1];
+	OS.memmove (pos, position, 4);
+	if (pos [0] == -1) {
+		int /*long*/ ptr = OS.gtk_entry_get_text (handle);
+		pos [0] = (int)/*64*/OS.g_utf8_strlen (ptr, -1);
+	}
+	/* Use the selection when the text was deleted */
+	int start = pos [0], end = pos [0];
+	if (fixStart != -1 && fixEnd != -1) {
+		start = pos [0] = fixStart;
+		end = fixEnd;
+		fixStart = fixEnd = -1;
+	}
+	String newText = verifyText (oldText, start, end);
+	if (newText != oldText) {
+		int [] newStart = new int [1], newEnd = new int [1];
 		OS.gtk_editable_get_selection_bounds (handle, newStart, newEnd);
-		boolean newSelection = oldStart [0] != newStart [0] || oldEnd [0] != newEnd [0];
-		if (newSelection) {
-			if (newText == null) newText = "";
-			pos [0] = newEnd [0];
-		}
-		if (newText == null) {
-			OS.g_signal_stop_emission_by_name (handle, OS.insert_text);
-		} else {
-			if (newText != oldText || newSelection) {
-				byte [] buffer3 = Converter.wcsToMbcs (null, newText, false);
-				OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-				if (newSelection) {
-					OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
-					OS.gtk_editable_delete_selection (handle);
-					OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
-				}
-				OS.gtk_editable_insert_text (handle, buffer3, buffer3.length, pos);
-				OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-				OS.g_signal_stop_emission_by_name (handle, OS.insert_text);
-				OS.memmove (position, pos, 4);
+		if (newText != null) {
+			if (newStart [0] != newEnd [0]) {
+				OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
+				OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
+				OS.gtk_editable_delete_selection (handle);
+				OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
+				OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 			}
+			byte [] buffer3 = Converter.wcsToMbcs (null, newText, false);
+			OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+			OS.gtk_editable_insert_text (handle, buffer3, buffer3.length, pos);
+			OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+			newStart [0] = newEnd [0] = pos [0];
 		}
-	} else {
-		byte [] iter = new byte [ITER_SIZEOF];
-		OS.memmove (iter, new_text, iter.length);
-		int start = OS.gtk_text_iter_get_offset (iter);
-		byte [] buffer = new byte [(int)/*64*/position];
-		OS.memmove (buffer, new_text_length, buffer.length);
-		String oldText = new String (Converter.mbcsToWcs (null, buffer));
-		String newText = verifyText (oldText, start, start);
-		if (newText == null) {
-			OS.g_signal_stop_emission_by_name (bufferHandle, OS.insert_text);
-		} else {
-			if (newText != oldText) {
-				byte [] buffer1 = Converter.wcsToMbcs (null, newText, false);
-				OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-				OS.gtk_text_buffer_insert (bufferHandle, new_text, buffer1, buffer1.length);
-				OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-				OS.g_signal_stop_emission_by_name (bufferHandle, OS.insert_text);
-			}
+		pos [0] = newEnd [0];
+		if (newStart [0] != newEnd [0]) {
+			fixStart = newStart [0];
+			fixEnd = newEnd [0];
 		}
+		OS.memmove (position, pos, 4);
+		OS.g_signal_stop_emission_by_name (handle, OS.insert_text);
 	}
 	return 0;
 }
@@ -1114,31 +1328,61 @@ int /*long*/ gtk_key_press_event (int /*long*/ widget, int /*long*/ event) {
 	return result;
 }
 
-int /*long*/ gtk_popup_menu (int /*long*/ widget) {
-	int [] x = new int [1], y = new int [1];
-	OS.gdk_window_get_pointer (0, x, y, null);
-	return showMenu (x [0], y [0]) ? 1 : 0;
+int /*long*/ gtk_populate_popup (int /*long*/ widget, int /*long*/ menu) {
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		OS.gtk_widget_set_direction (menu, OS.GTK_TEXT_DIR_RTL);
+		OS.gtk_container_forall (menu, display.setDirectionProc, OS.GTK_TEXT_DIR_RTL);
+	}
+	return 0;
+}
+
+int /*long*/ gtk_text_buffer_insert_text (int /*long*/ widget, int /*long*/ iter, int /*long*/ text, int /*long*/ length) {
+	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
+	byte [] position = new byte [ITER_SIZEOF];
+	OS.memmove (position, iter, position.length);
+	/* Use the selection when the text was deleted */
+	int start = OS.gtk_text_iter_get_offset (position), end = start;
+	if (fixStart != -1 && fixEnd != -1) {
+		start = fixStart;
+		end = fixEnd;
+		fixStart = fixEnd = -1;
+	}
+	byte [] buffer = new byte [(int)/*64*/length];
+	OS.memmove (buffer, text, buffer.length);
+	String oldText = new String (Converter.mbcsToWcs (null, buffer));
+	String newText = verifyText (oldText, start, end);
+	if (newText == null) {
+		OS.g_signal_stop_emission_by_name (bufferHandle, OS.insert_text);
+	} else {
+		if (newText != oldText) {
+			byte [] buffer1 = Converter.wcsToMbcs (null, newText, false);
+			OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
+			OS.gtk_text_buffer_insert (bufferHandle, iter, buffer1, buffer1.length);
+			OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
+			OS.g_signal_stop_emission_by_name (bufferHandle, OS.insert_text);
+		}
+	}
+	return 0;
 }
 
 void hookEvents () {
 	super.hookEvents();
-	int /*long*/ windowProc2 = display.windowProc2;
-	int /*long*/ windowProc3 = display.windowProc3;
-	int /*long*/ windowProc4 = display.windowProc4;
-	int /*long*/ windowProc5 = display.windowProc5;
 	if ((style & SWT.SINGLE) != 0) {
-		OS.g_signal_connect_after (handle, OS.changed, windowProc2, CHANGED);
-		OS.g_signal_connect (handle, OS.insert_text, windowProc5, INSERT_TEXT);
-		OS.g_signal_connect (handle, OS.delete_text, windowProc4, DELETE_TEXT);
-		OS.g_signal_connect (handle, OS.activate, windowProc2, ACTIVATE);
+		OS.g_signal_connect_closure (handle, OS.changed, display.closures [CHANGED], true);
+		OS.g_signal_connect_closure (handle, OS.insert_text, display.closures [INSERT_TEXT], false);
+		OS.g_signal_connect_closure (handle, OS.delete_text, display.closures [DELETE_TEXT], false);
+		OS.g_signal_connect_closure (handle, OS.activate, display.closures [ACTIVATE], false);
+		OS.g_signal_connect_closure (handle, OS.grab_focus, display.closures [GRAB_FOCUS], false);
+		OS.g_signal_connect_closure (handle, OS.populate_popup, display.closures [POPULATE_POPUP], false);
 	} else {
-		OS.g_signal_connect (bufferHandle, OS.changed, windowProc2, CHANGED);
-		OS.g_signal_connect (bufferHandle, OS.insert_text, windowProc5, INSERT_TEXT);
-		OS.g_signal_connect (bufferHandle, OS.delete_range, windowProc4, DELETE_RANGE);
+		OS.g_signal_connect_closure (bufferHandle, OS.changed, display.closures [CHANGED], false);
+		OS.g_signal_connect_closure (bufferHandle, OS.insert_text, display.closures [TEXT_BUFFER_INSERT_TEXT], false);
+		OS.g_signal_connect_closure (bufferHandle, OS.delete_range, display.closures [DELETE_RANGE], false);
+		OS.g_signal_connect_closure (handle, OS.populate_popup, display.closures [POPULATE_POPUP], false);
 	}
 	int /*long*/ imContext = imContext ();
 	if (imContext != 0) {
-		OS.g_signal_connect (imContext, OS.commit, windowProc3, COMMIT);
+		OS.g_signal_connect_closure (imContext, OS.commit, display.closures [COMMIT], false);
 		int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
 		int mask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
 		OS.g_signal_handlers_block_matched (imContext, mask, id, 0, 0, 0, handle);
@@ -1146,7 +1390,10 @@ void hookEvents () {
 }
 
 int /*long*/ imContext () {
-	return (style & SWT.SINGLE) != 0 ? OS.GTK_ENTRY_IM_CONTEXT (handle) : OS.GTK_TEXTVIEW_IM_CONTEXT (handle);
+	if ((style & SWT.SINGLE) != 0) {
+		return OS.gtk_editable_get_editable (handle) ? OS.GTK_ENTRY_IM_CONTEXT (handle) : 0;
+	} 
+	return OS.GTK_TEXTVIEW_IM_CONTEXT (handle);
 }
 
 /**
@@ -1230,8 +1477,8 @@ void register () {
 }
 
 void releaseWidget () {
-	fixIM ();	
 	super.releaseWidget ();
+	fixIM ();	
 	if (OS.GTK_VERSION < OS.VERSION (2, 6, 0)) {		 
 		/*
 		* Bug in GTK.  Any text copied into the clipboard will be lost when
@@ -1244,6 +1491,7 @@ void releaseWidget () {
 			OS.gtk_text_buffer_paste_clipboard (bufferHandle, clipboard, null, OS.gtk_text_view_get_editable (handle));
 		}
 	}
+	message = null;
 }
 
 /**
@@ -1272,7 +1520,7 @@ public void removeModifyListener (ModifyListener listener) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notified when the control is selected.
+ * be notified when the control is selected by the user.
  *
  * @param listener the listener which should no longer be notified
  *
@@ -1433,9 +1681,32 @@ void setFontDescription (int /*long*/ font) {
 	setTabStops (tabs);
 }
 
-void setForegroundColor (GdkColor color) {
-	super.setForegroundColor (color);
-	OS.gtk_widget_modify_text (handle, 0, color);
+/**
+ * Sets the widget message. When the widget is created
+ * with the style <code>SWT.SEARCH</code>, the message text
+ * is displayed as a hint for the user, indicating the
+ * purpose of the field.
+ * <p>
+ * Note: This operation is a <em>HINT</em> and is not
+ * supported on platforms that do not have this concept.
+ * </p>
+ * 
+ * @param message the new message
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the message is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.3
+ */
+public void setMessage (String message) {
+	checkWidget ();
+	if (message == null) error (SWT.ERROR_NULL_ARGUMENT);
+	this.message = message;
 }
 
 /**
@@ -1644,11 +1915,11 @@ public void setText (String string) {
 		byte [] position =  new byte [ITER_SIZEOF];
 		OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 		OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_RANGE);
-		OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+		OS.g_signal_handlers_block_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
 		OS.gtk_text_buffer_set_text (bufferHandle, buffer, buffer.length);
 		OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 		OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_RANGE);
-		OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+		OS.g_signal_handlers_unblock_matched (bufferHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEXT_BUFFER_INSERT_TEXT);
 		OS.gtk_text_buffer_get_iter_at_offset (bufferHandle, position, 0);
 		OS.gtk_text_buffer_place_cursor (bufferHandle, position);
 		int /*long*/ mark = OS.gtk_text_buffer_get_insert (bufferHandle);
