@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,6 +32,9 @@ import org.eclipse.swt.events.*;
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
+ *
+ * @see <a href="http://www.eclipse.org/swt/snippets/#toolbar">ToolBar, ToolItem snippets</a>
+ * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public class ToolItem extends Item {
 	int /*long*/ boxHandle, arrowHandle, arrowBoxHandle, separatorHandle, labelHandle, imageHandle;
@@ -98,10 +101,11 @@ public ToolItem (ToolBar parent, int style) {
  *
  * @param parent a composite control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -121,14 +125,14 @@ public ToolItem (ToolBar parent, int style, int index) {
 	this.parent = parent;
 	int count = parent.getItemCount ();
 	if (!(0 <= index && index <= count)) {
-		error (SWT.ERROR_ITEM_NOT_ADDED);
+		error (SWT.ERROR_INVALID_RANGE);
 	}
 	createWidget (index);
 }
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the control is selected by the user, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -137,7 +141,7 @@ public ToolItem (ToolBar parent, int style, int index) {
  * <code>widgetDefaultSelected</code> is not called.
  * </p>
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should be notified when the control is selected by the user,
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -178,6 +182,11 @@ void createHandle (int index) {
 		if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 		OS.gtk_container_add (boxHandle, imageHandle);
 		OS.gtk_container_add (boxHandle, labelHandle);
+		if ((parent.style & SWT.VERTICAL) != 0) {
+			// Align text and images to the left
+			OS.gtk_box_set_child_packing (boxHandle, imageHandle, false, false, 0, OS.GTK_PACK_START);
+			OS.gtk_box_set_child_packing (boxHandle, labelHandle, false, false, 2, OS.GTK_PACK_START);
+		}
 	}
 	int bits = SWT.SEPARATOR | SWT.RADIO | SWT.CHECK | SWT.PUSH | SWT.DROP_DOWN;
 	switch (style & bits) {
@@ -284,16 +293,45 @@ public Rectangle getBounds () {
 	checkWidget();
 	parent.forceResize ();
 	int /*long*/ topHandle = topHandle ();
-	int x = OS.GTK_WIDGET_X (topHandle);
-	int y = OS.GTK_WIDGET_Y (topHandle);
-	int width = OS.GTK_WIDGET_WIDTH (topHandle);
-	int height = OS.GTK_WIDGET_HEIGHT (topHandle);
+	int x, y, width, height;
+	/*
+	* Bug in GTK.  Toolbar items are only allocated their minimum size
+	* in versions before 2.4.0.  The fix is to use the total size
+	* available minus any borders.
+	*/
+	if (OS.GTK_VERSION < OS.VERSION (2, 4, 0) && control != null && !control.isDisposed ()) {
+		int border = OS.gtk_container_get_border_width (parent.handle);
+		byte [] shadowType = Converter.wcsToMbcs (null, "shadow_type", true);
+		int [] shadow = new int [1];
+		OS.gtk_widget_style_get (parent.handle, shadowType, shadow, 0);
+		if (shadow [0] != OS.GTK_SHADOW_NONE) {
+			border += OS.gtk_style_get_xthickness (OS.gtk_widget_get_style (parent.handle));
+		}
+		if ((parent.style & SWT.VERTICAL) != 0) {
+			x = border;
+			y = OS.GTK_WIDGET_Y (topHandle) + border;
+			width = OS.GTK_WIDGET_WIDTH (parent.handle) - border*2;
+			height = OS.GTK_WIDGET_HEIGHT (topHandle);			
+		} else {
+			x = OS.GTK_WIDGET_X (topHandle) + border;
+			y = border;
+			width = OS.GTK_WIDGET_WIDTH (topHandle);
+			height = OS.GTK_WIDGET_HEIGHT (parent.handle) - border*2;
+		}
+	} else {
+		x = OS.GTK_WIDGET_X (topHandle);
+		y = OS.GTK_WIDGET_Y (topHandle);
+		width = OS.GTK_WIDGET_WIDTH (topHandle);
+		height = OS.GTK_WIDGET_HEIGHT (topHandle);		
+	}
+	if ((parent.style & SWT.MIRRORED) != 0) x = parent.getClientWidth () - width - x;
+	if ((style & SWT.SEPARATOR) != 0 && control != null) height = Math.max (height, 23);
 	return new Rectangle (x, y, width, height);
 }
 
 /**
  * Returns the control that is used to fill the bounds of
- * the item when the items is a <code>SEPARATOR</code>.
+ * the item when the item is a <code>SEPARATOR</code>.
  *
  * @return the control
  *
@@ -438,33 +476,33 @@ public int getWidth () {
 }
 
 int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
-	GdkEventButton gdkEvent = new GdkEventButton();
-	OS.memmove(gdkEvent, event, GdkEventButton.sizeof);
+	GdkEventButton gdkEvent = new GdkEventButton ();
+	OS.memmove (gdkEvent, event, GdkEventButton.sizeof);
 	double x = gdkEvent.x;
-	gdkEvent.x += OS.GTK_WIDGET_X(handle);
+	gdkEvent.x += OS.GTK_WIDGET_X (handle);
 	double y = gdkEvent.y;
-	gdkEvent.y += OS.GTK_WIDGET_Y(handle);
-	OS.memmove(event, gdkEvent, GdkEventButton.sizeof);
-	parent.gtk_button_press_event (widget, event);
+	gdkEvent.y += OS.GTK_WIDGET_Y (handle);
+	OS.memmove (event, gdkEvent, GdkEventButton.sizeof);
+	int /*long*/ result = parent.gtk_button_press_event (widget, event);
 	gdkEvent.x = x;
 	gdkEvent.y = y;
-	OS.memmove(event, gdkEvent, GdkEventButton.sizeof);
-	return 0;
+	OS.memmove (event, gdkEvent, GdkEventButton.sizeof);
+	return result;
 }
 
 int /*long*/ gtk_button_release_event (int /*long*/ widget, int /*long*/ event) {
-	GdkEventButton gdkEvent = new GdkEventButton();
-	OS.memmove(gdkEvent, event, GdkEventButton.sizeof);
+	GdkEventButton gdkEvent = new GdkEventButton ();
+	OS.memmove (gdkEvent, event, GdkEventButton.sizeof);
 	double x = gdkEvent.x;
-	gdkEvent.x += OS.GTK_WIDGET_X(handle);
+	gdkEvent.x += OS.GTK_WIDGET_X (handle);
 	double y = gdkEvent.y;
-	gdkEvent.y += OS.GTK_WIDGET_Y(handle);
-	OS.memmove(event, gdkEvent, GdkEventButton.sizeof);
-	parent.gtk_button_release_event (widget, event);
+	gdkEvent.y += OS.GTK_WIDGET_Y (handle);
+	OS.memmove (event, gdkEvent, GdkEventButton.sizeof);
+	int /*long*/ result = parent.gtk_button_release_event (widget, event);
 	gdkEvent.x = x;
 	gdkEvent.y = y;
 	OS.memmove(event, gdkEvent, GdkEventButton.sizeof);
-	return 0;
+	return result;
 }
 
 int /*long*/ gtk_clicked (int /*long*/ widget) {
@@ -483,10 +521,12 @@ int /*long*/ gtk_clicked (int /*long*/ widget) {
 					OS.gdk_event_get_coords (eventPtr, x_win, y_win);
 					int x = OS.GTK_WIDGET_X (arrowHandle) - OS.GTK_WIDGET_X (handle);
 					int width = OS.GTK_WIDGET_WIDTH (arrowHandle);
-					if (x <= (int)x_win [0] && (int)x_win [0] <= x + width) {
+					if ((((parent.style & SWT.RIGHT_TO_LEFT) == 0) && x <= (int)x_win [0])
+						|| (((parent.style & SWT.RIGHT_TO_LEFT) != 0) && (int)x_win [0] <= x + width)) {
 						event.detail = SWT.ARROW;
 						int /*long*/ topHandle = topHandle ();
 						event.x = OS.GTK_WIDGET_X (topHandle);
+						if ((parent.style & SWT.MIRRORED) != 0) event.x = parent.getClientWidth () - OS.GTK_WIDGET_WIDTH (topHandle) - event.x;
 						event.y = OS.GTK_WIDGET_Y (topHandle) + OS.GTK_WIDGET_HEIGHT (topHandle);
 					}
 					break;
@@ -505,6 +545,7 @@ int /*long*/ gtk_clicked (int /*long*/ widget) {
 }
 
 int /*long*/ gtk_enter_notify_event (int /*long*/ widget, int /*long*/ event) {
+	parent.gtk_enter_notify_event (widget, event);
 	drawHotImage = (parent.style & SWT.FLAT) != 0 && hotImage != null;
 	if (drawHotImage && imageHandle != 0) {
 		ImageList imageList = parent.imageList;
@@ -542,6 +583,7 @@ int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
 }
 
 int /*long*/ gtk_leave_notify_event (int /*long*/ widget, int /*long*/ event) {
+	parent.gtk_leave_notify_event (widget, event);
 	if (drawHotImage) {
 		drawHotImage = false;
 		if (imageHandle != 0 && image != null) {
@@ -574,14 +616,12 @@ boolean hasFocus () {
 void hookEvents () {
 	super.hookEvents ();
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int /*long*/ windowProc2 = display.windowProc2;
-	int /*long*/ windowProc3 = display.windowProc3;
-	OS.g_signal_connect (handle, OS.clicked, windowProc2, CLICKED);
-	OS.g_signal_connect (handle, OS.enter_notify_event, windowProc3, ENTER_NOTIFY_EVENT);
-	OS.g_signal_connect (handle, OS.leave_notify_event, windowProc3, LEAVE_NOTIFY_EVENT);
-	if (labelHandle != 0) OS.g_signal_connect (labelHandle, OS.mnemonic_activate, display.windowProc3, MNEMONIC_ACTIVATE);
+	OS.g_signal_connect_closure (handle, OS.clicked, display.closures [CLICKED], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [ENTER_NOTIFY_EVENT], 0, display.closures [ENTER_NOTIFY_EVENT], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [LEAVE_NOTIFY_EVENT], 0, display.closures [LEAVE_NOTIFY_EVENT], false);
+	if (labelHandle != 0) OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.closures [MNEMONIC_ACTIVATE], false);
 
-	OS.g_signal_connect (handle, OS.focus_out_event, windowProc3, FOCUS_OUT_EVENT);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [FOCUS_OUT_EVENT], 0, display.closures [FOCUS_OUT_EVENT], false);
 
 	/*
 	* Feature in GTK.  Usually, GTK widgets propagate all events to their
@@ -598,12 +638,12 @@ void hookEvents () {
 		OS.GDK_KEY_PRESS_MASK | OS.GDK_KEY_RELEASE_MASK |
 		OS.GDK_FOCUS_CHANGE_MASK;
 	OS.gtk_widget_add_events (handle, mask);
-	OS.g_signal_connect (handle, OS.button_press_event, windowProc3, BUTTON_PRESS_EVENT);
-	OS.g_signal_connect (handle, OS.button_release_event, windowProc3, BUTTON_RELEASE_EVENT);
-	OS.g_signal_connect (handle, OS.event_after, windowProc3, EVENT_AFTER);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [BUTTON_PRESS_EVENT], 0, display.closures [BUTTON_PRESS_EVENT], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [BUTTON_RELEASE_EVENT], 0, display.closures [BUTTON_RELEASE_EVENT], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [EVENT_AFTER], 0, display.closures[EVENT_AFTER], false);
 
 	int /*long*/ topHandle = topHandle ();
-	OS.g_signal_connect_after (topHandle, OS.map, windowProc2, MAP);
+	OS.g_signal_connect_closure_by_id (topHandle, display.signalIds [MAP], 0, display.closures [MAP], true);
 }
 
 /**
@@ -637,8 +677,6 @@ void releaseHandle () {
 }
 
 void releaseWidget () {
-	/* Reparent the control back to the toolbar */
-	if (control != null) setControl (null);
 	super.releaseWidget ();
 	if (parent.lastFocus == this) parent.lastFocus = null;
 	parent = null;
@@ -649,7 +687,7 @@ void releaseWidget () {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notified when the control is selected.
+ * be notified when the control is selected by the user.
  *
  * @param listener the listener which should no longer be notified
  *
@@ -685,6 +723,7 @@ void resizeControl () {
 		*/
 		Rectangle itemRect = getBounds ();
 		control.setSize (itemRect.width, itemRect.height);
+		OS.gtk_widget_set_size_request (handle, itemRect.width, itemRect.height);
 		Rectangle rect = control.getBounds ();
 		rect.x = itemRect.x + (itemRect.width - rect.width) / 2;
 		rect.y = itemRect.y + (itemRect.height - rect.height) / 2;
@@ -707,7 +746,7 @@ void selectRadio () {
 
 /**
  * Sets the control that is used to fill the bounds of
- * the item when the items is a <code>SEPARATOR</code>.
+ * the item when the item is a <code>SEPARATOR</code>.
  *
  * @param control the new control
  *
@@ -736,7 +775,7 @@ public void setControl (Control control) {
  * Sets the receiver's disabled image to the argument, which may be
  * null indicating that no disabled image should be displayed.
  * <p>
- * The disbled image is displayed when the receiver is disabled.
+ * The disabled image is displayed when the receiver is disabled.
  * </p>
  *
  * @param image the disabled image to display on the receiver (may be null)
@@ -808,9 +847,9 @@ void setFontDescription (int /*long*/ font) {
 }
 
 void setForegroundColor (GdkColor color) {
-	OS.gtk_widget_modify_fg (handle,  OS.GTK_STATE_NORMAL, color);
-	if (labelHandle != 0) OS.gtk_widget_modify_fg (labelHandle,  OS.GTK_STATE_NORMAL, color);
-	if (imageHandle != 0) OS.gtk_widget_modify_fg (imageHandle,  OS.GTK_STATE_NORMAL, color);
+	setForegroundColor (handle, color);
+	if (labelHandle != 0) setForegroundColor (labelHandle, color);
+	if (imageHandle != 0) setForegroundColor (imageHandle, color);
 }
 
 /**
@@ -907,8 +946,7 @@ boolean setRadioSelection (boolean value) {
  * </ul>
  */
 public void setSelection (boolean selected) {
-	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (!isValidWidget ()) error (SWT.ERROR_WIDGET_DISPOSED);
+	checkWidget ();
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CLICKED);
 	OS.gtk_toggle_button_set_active (handle, selected);
@@ -920,14 +958,14 @@ public void setSelection (boolean selected) {
  * the mnemonic character.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
@@ -970,11 +1008,15 @@ public void setText (String string) {
  */
 public void setToolTipText (String string) {
 	checkWidget();
-	toolTipText = string;
 	if (parent.toolTipText == null) {
 		Shell shell = parent._getShell ();
-		shell.setToolTipText (handle, toolTipText);
+		setToolTipText (shell, string);
 	}
+	toolTipText = string;
+}
+
+void setToolTipText (Shell shell, String newString) {
+	shell.setToolTipText (handle, newString);
 }
 
 /**
@@ -991,7 +1033,9 @@ public void setWidth (int width) {
 	checkWidget();
 	if ((style & SWT.SEPARATOR) == 0) return;
 	if (width < 0) return;
-	OS.gtk_widget_set_size_request (handle, width, -1);
+	boolean isVertical = (parent.style & SWT.VERTICAL) != 0;
+	OS.gtk_widget_set_size_request (separatorHandle, width, isVertical ? 6 : 15);
+	OS.gtk_widget_set_size_request (handle, width, isVertical ? 6 : 15);
 	parent.relayout ();
 }
 
