@@ -1,27 +1,29 @@
 /*******************************************************************************
-* Copyright (c) 2000, 2005 IBM Corporation and others. All rights reserved.
-* The contents of this file are made available under the terms
-* of the GNU Lesser General Public License (LGPL) Version 2.1 that
-* accompanies this distribution (lgpl-v21.txt).  The LGPL is also
-* available at http://www.gnu.org/licenses/lgpl.html.  If the version
-* of the LGPL at http://www.gnu.org is different to the version of
-* the LGPL accompanying this distribution and there is any conflict
-* between the two license versions, the terms of the LGPL accompanying
-* this distribution shall govern.
-* 
-* Contributors:
-*     IBM Corporation - initial API and implementation
-*******************************************************************************/
+ * Copyright (c) 2000, 2008 IBM Corporation and others. All rights reserved.
+ * The contents of this file are made available under the terms
+ * of the GNU Lesser General Public License (LGPL) Version 2.1 that
+ * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
+ * available at http://www.gnu.org/licenses/lgpl.html.  If the version
+ * of the LGPL at http://www.gnu.org is different to the version of
+ * the LGPL accompanying this distribution and there is any conflict
+ * between the two license versions, the terms of the LGPL accompanying
+ * this distribution shall govern.
+ * 
+ * Contributors:
+ *     IBM Corporation - initial API and implementation
+ *******************************************************************************/
 
 #include "swt.h"
 #include "os_stats.h"
 
 #ifdef NATIVE_STATS
 
-int OS_nativeFunctionCount = 992;
-int OS_nativeFunctionCallCount[992];
+int OS_nativeFunctionCount = 1265;
+int OS_nativeFunctionCallCount[1265];
 char * OS_nativeFunctionNames[] = {
 	"Call",
+	"GDK_1EVENT_1TYPE",
+	"GDK_1EVENT_1WINDOW",
 	"GDK_1WINDOWING_1X11",
 	"GInterfaceInfo_1sizeof",
 	"GPollFD_1sizeof",
@@ -32,6 +34,8 @@ char * OS_nativeFunctionNames[] = {
 	"GTK_1SCROLLED_1WINDOW_1SCROLLBAR_1SPACING",
 	"GTK_1SCROLLED_1WINDOW_1VSCROLLBAR",
 	"GTK_1TEXTVIEW_1IM_1CONTEXT",
+	"GTK_1TOOLTIPS_1SET_1ACTIVE",
+	"GTK_1TOOLTIPS_1TIP_1WINDOW",
 	"GTK_1WIDGET_1HEIGHT",
 	"GTK_1WIDGET_1REQUISITION_1HEIGHT",
 	"GTK_1WIDGET_1REQUISITION_1WIDTH",
@@ -47,6 +51,7 @@ char * OS_nativeFunctionNames[] = {
 	"GTypeQuery_1sizeof",
 	"GdkColor_1sizeof",
 	"GdkDragContext_1sizeof",
+	"GdkEventAny_1sizeof",
 	"GdkEventButton_1sizeof",
 	"GdkEventCrossing_1sizeof",
 	"GdkEventExpose_1sizeof",
@@ -66,6 +71,12 @@ char * OS_nativeFunctionNames[] = {
 	"GtkAdjustment_1sizeof",
 	"GtkAllocation_1sizeof",
 	"GtkBorder_1sizeof",
+	"GtkCellRendererPixbufClass_1sizeof",
+	"GtkCellRendererPixbuf_1sizeof",
+	"GtkCellRendererTextClass_1sizeof",
+	"GtkCellRendererText_1sizeof",
+	"GtkCellRendererToggleClass_1sizeof",
+	"GtkCellRendererToggle_1sizeof",
 	"GtkColorSelectionDialog_1sizeof",
 	"GtkCombo_1sizeof",
 	"GtkFileSelection_1sizeof",
@@ -77,7 +88,8 @@ char * OS_nativeFunctionNames[] = {
 	"GtkTargetPair_1sizeof",
 	"GtkTextIter_1sizeof",
 	"GtkTreeIter_1sizeof",
-	"PTR_1sizeof",
+	"PangoAttrColor_1sizeof",
+	"PangoAttrInt_1sizeof",
 	"PangoAttribute_1sizeof",
 	"PangoItem_1sizeof",
 	"PangoLayoutLine_1sizeof",
@@ -91,9 +103,13 @@ char * OS_nativeFunctionNames[] = {
 	"XEvent_1sizeof",
 	"XExposeEvent_1sizeof",
 	"XFocusChangeEvent_1sizeof",
+	"XRenderPictureAttributes_1sizeof",
 	"XVisibilityEvent_1sizeof",
 	"XWindowChanges_1sizeof",
+	"X_1EVENT_1TYPE",
+	"X_1EVENT_1WINDOW",
 	"_1Call",
+	"_1FcConfigAppFontAddFile",
 	"_1GDK_1DISPLAY",
 	"_1GDK_1PIXMAP_1XID",
 	"_1GDK_1ROOT_1PARENT",
@@ -102,11 +118,20 @@ char * OS_nativeFunctionNames[] = {
 	"_1GTK_1IS_1BUTTON",
 	"_1GTK_1IS_1CELL_1RENDERER_1PIXBUF",
 	"_1GTK_1IS_1CELL_1RENDERER_1TEXT",
+	"_1GTK_1IS_1CELL_1RENDERER_1TOGGLE",
 	"_1GTK_1IS_1CONTAINER",
 	"_1GTK_1IS_1IMAGE_1MENU_1ITEM",
+	"_1GTK_1IS_1MENU_1ITEM",
+	"_1GTK_1IS_1PLUG",
+	"_1GTK_1IS_1WINDOW",
 	"_1GTK_1STOCK_1CANCEL",
 	"_1GTK_1STOCK_1OK",
+	"_1GTK_1TYPE_1CELL_1RENDERER_1PIXBUF",
+	"_1GTK_1TYPE_1CELL_1RENDERER_1TEXT",
+	"_1GTK_1TYPE_1CELL_1RENDERER_1TOGGLE",
 	"_1GTK_1TYPE_1FIXED",
+	"_1GTK_1TYPE_1MENU",
+	"_1GTK_1TYPE_1WIDGET",
 	"_1GTK_1WIDGET_1FLAGS",
 	"_1GTK_1WIDGET_1HAS_1DEFAULT",
 	"_1GTK_1WIDGET_1HAS_1FOCUS",
@@ -120,6 +145,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1G_1OBJECT_1CLASS",
 	"_1G_1OBJECT_1GET_1CLASS",
 	"_1G_1OBJECT_1TYPE",
+	"_1G_1OBJECT_1TYPE_1NAME",
 	"_1G_1TYPE_1BOOLEAN",
 	"_1G_1TYPE_1INT",
 	"_1G_1TYPE_1STRING",
@@ -136,8 +162,18 @@ char * OS_nativeFunctionNames[] = {
 	"_1XInternAtom",
 	"_1XKeysymToKeycode",
 	"_1XListProperties",
+	"_1XQueryPointer",
 	"_1XQueryTree",
 	"_1XReconfigureWMWindow",
+	"_1XRenderComposite",
+	"_1XRenderCreatePicture",
+	"_1XRenderFindStandardFormat",
+	"_1XRenderFindVisualFormat",
+	"_1XRenderFreePicture",
+	"_1XRenderQueryExtension",
+	"_1XRenderQueryVersion",
+	"_1XRenderSetPictureClipRectangles",
+	"_1XRenderSetPictureTransform",
 	"_1XSendEvent",
 	"_1XSetErrorHandler",
 	"_1XSetIOErrorHandler",
@@ -147,11 +183,20 @@ char * OS_nativeFunctionNames[] = {
 	"_1XTestFakeKeyEvent",
 	"_1XTestFakeMotionEvent",
 	"_1XWarpPointer",
+	"_1atk_1object_1add_1relationship",
+	"_1call",
+	"_1dlclose",
+	"_1dlopen",
+	"_1dlsym",
+	"_1g_1cclosure_1new",
+	"_1g_1closure_1ref",
+	"_1g_1closure_1unref",
 	"_1g_1filename_1from_1uri",
 	"_1g_1filename_1from_1utf8",
 	"_1g_1filename_1to_1uri",
 	"_1g_1filename_1to_1utf8",
 	"_1g_1free",
+	"_1g_1idle_1add",
 	"_1g_1list_1append",
 	"_1g_1list_1data",
 	"_1g_1list_1free",
@@ -188,13 +233,17 @@ char * OS_nativeFunctionNames[] = {
 	"_1g_1object_1set__I_3BFI",
 	"_1g_1object_1set__I_3BII",
 	"_1g_1object_1set__I_3BJI",
+	"_1g_1object_1set__I_3BLorg_eclipse_swt_internal_gtk_GdkColor_2I",
 	"_1g_1object_1set__I_3BZI",
 	"_1g_1object_1set_1qdata",
 	"_1g_1object_1unref",
 	"_1g_1quark_1from_1string",
 	"_1g_1set_1prgname",
+	"_1g_1signal_1add_1emission_1hook",
 	"_1g_1signal_1connect",
 	"_1g_1signal_1connect_1after",
+	"_1g_1signal_1connect_1closure",
+	"_1g_1signal_1connect_1closure_1by_1id",
 	"_1g_1signal_1emit_1by_1name__I_3B",
 	"_1g_1signal_1emit_1by_1name__I_3BI",
 	"_1g_1signal_1emit_1by_1name__I_3BII",
@@ -204,12 +253,15 @@ char * OS_nativeFunctionNames[] = {
 	"_1g_1signal_1handlers_1disconnect_1matched",
 	"_1g_1signal_1handlers_1unblock_1matched",
 	"_1g_1signal_1lookup",
+	"_1g_1signal_1remove_1emission_1hook",
 	"_1g_1signal_1stop_1emission_1by_1name",
 	"_1g_1slist_1data",
 	"_1g_1slist_1free",
 	"_1g_1slist_1length",
 	"_1g_1slist_1next",
+	"_1g_1source_1remove",
 	"_1g_1strfreev",
+	"_1g_1strtod",
 	"_1g_1thread_1init",
 	"_1g_1thread_1supported",
 	"_1g_1type_1add_1interface_1static",
@@ -228,10 +280,13 @@ char * OS_nativeFunctionNames[] = {
 	"_1g_1utf8_1strlen",
 	"_1g_1utf8_1to_1utf16__II_3I_3I_3I",
 	"_1g_1utf8_1to_1utf16___3BI_3I_3I_3I",
+	"_1g_1value_1peek_1pointer",
 	"_1gdk_1atom_1intern",
 	"_1gdk_1atom_1name",
 	"_1gdk_1beep",
 	"_1gdk_1bitmap_1create_1from_1data",
+	"_1gdk_1cairo_1region",
+	"_1gdk_1cairo_1set_1source_1color",
 	"_1gdk_1color_1white",
 	"_1gdk_1colormap_1alloc_1color",
 	"_1gdk_1colormap_1free_1colors",
@@ -239,10 +294,14 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1colormap_1query_1color",
 	"_1gdk_1cursor_1destroy",
 	"_1gdk_1cursor_1new",
+	"_1gdk_1cursor_1new_1from_1pixbuf",
 	"_1gdk_1cursor_1new_1from_1pixmap",
+	"_1gdk_1display_1get_1default",
+	"_1gdk_1display_1supports_1cursor_1color",
 	"_1gdk_1drag_1status",
 	"_1gdk_1draw_1arc",
 	"_1gdk_1draw_1drawable",
+	"_1gdk_1draw_1image",
 	"_1gdk_1draw_1layout",
 	"_1gdk_1draw_1layout_1with_1colors",
 	"_1gdk_1draw_1line",
@@ -251,6 +310,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1draw_1point",
 	"_1gdk_1draw_1polygon",
 	"_1gdk_1draw_1rectangle",
+	"_1gdk_1drawable_1get_1depth",
 	"_1gdk_1drawable_1get_1image",
 	"_1gdk_1drawable_1get_1size",
 	"_1gdk_1drawable_1get_1visible_1region",
@@ -265,6 +325,8 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1event_1get_1state",
 	"_1gdk_1event_1get_1time",
 	"_1gdk_1event_1handler_1set",
+	"_1gdk_1event_1new",
+	"_1gdk_1event_1peek",
 	"_1gdk_1event_1put",
 	"_1gdk_1flush",
 	"_1gdk_1free_1text_1list",
@@ -283,9 +345,9 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1gc_1set_1line_1attributes",
 	"_1gdk_1gc_1set_1stipple",
 	"_1gdk_1gc_1set_1subwindow",
+	"_1gdk_1gc_1set_1tile",
+	"_1gdk_1gc_1set_1ts_1origin",
 	"_1gdk_1gc_1set_1values",
-	"_1gdk_1image_1get",
-	"_1gdk_1image_1get_1pixel",
 	"_1gdk_1keyboard_1ungrab",
 	"_1gdk_1keymap_1get_1default",
 	"_1gdk_1keymap_1translate_1keyboard_1state",
@@ -301,10 +363,16 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1pixbuf_1get_1pixels",
 	"_1gdk_1pixbuf_1get_1rowstride",
 	"_1gdk_1pixbuf_1get_1width",
+	"_1gdk_1pixbuf_1loader_1close",
+	"_1gdk_1pixbuf_1loader_1get_1pixbuf",
+	"_1gdk_1pixbuf_1loader_1new",
+	"_1gdk_1pixbuf_1loader_1write",
 	"_1gdk_1pixbuf_1new",
+	"_1gdk_1pixbuf_1new_1from_1file",
 	"_1gdk_1pixbuf_1render_1pixmap_1and_1mask",
 	"_1gdk_1pixbuf_1render_1to_1drawable",
 	"_1gdk_1pixbuf_1render_1to_1drawable_1alpha",
+	"_1gdk_1pixbuf_1save_1to_1buffer",
 	"_1gdk_1pixbuf_1scale",
 	"_1gdk_1pixbuf_1scale_1simple",
 	"_1gdk_1pixmap_1foreign_1new",
@@ -329,6 +397,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1region_1union_1with_1rect",
 	"_1gdk_1rgb_1init",
 	"_1gdk_1screen_1get_1default",
+	"_1gdk_1screen_1get_1monitor_1at_1point",
 	"_1gdk_1screen_1get_1monitor_1at_1window",
 	"_1gdk_1screen_1get_1monitor_1geometry",
 	"_1gdk_1screen_1get_1n_1monitors",
@@ -340,10 +409,12 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1text_1property_1to_1utf8_1list",
 	"_1gdk_1unicode_1to_1keyval",
 	"_1gdk_1utf8_1to_1compound_1text",
+	"_1gdk_1utf8_1to_1string_1target",
 	"_1gdk_1visual_1get_1system",
 	"_1gdk_1window_1add_1filter",
 	"_1gdk_1window_1at_1pointer",
 	"_1gdk_1window_1begin_1paint_1rect",
+	"_1gdk_1window_1clear_1area",
 	"_1gdk_1window_1destroy",
 	"_1gdk_1window_1end_1paint",
 	"_1gdk_1window_1focus",
@@ -355,10 +426,12 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1window_1get_1origin",
 	"_1gdk_1window_1get_1parent",
 	"_1gdk_1window_1get_1pointer",
+	"_1gdk_1window_1get_1position",
 	"_1gdk_1window_1get_1user_1data",
 	"_1gdk_1window_1hide",
 	"_1gdk_1window_1invalidate_1rect",
 	"_1gdk_1window_1invalidate_1region",
+	"_1gdk_1window_1is_1visible",
 	"_1gdk_1window_1lookup",
 	"_1gdk_1window_1lower",
 	"_1gdk_1window_1move",
@@ -372,6 +445,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1window_1set_1accept_1focus",
 	"_1gdk_1window_1set_1back_1pixmap",
 	"_1gdk_1window_1set_1cursor",
+	"_1gdk_1window_1set_1debug_1updates",
 	"_1gdk_1window_1set_1decorations",
 	"_1gdk_1window_1set_1events",
 	"_1gdk_1window_1set_1icon",
@@ -388,6 +462,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1x11_1drawable_1get_1xdisplay",
 	"_1gdk_1x11_1drawable_1get_1xid",
 	"_1gdk_1x11_1screen_1get_1window_1manager_1name",
+	"_1gdk_1x11_1screen_1lookup_1visual",
 	"_1gdk_1x11_1visual_1get_1xvisual",
 	"_1gtk_1accel_1group_1new",
 	"_1gtk_1accel_1groups_1activate",
@@ -400,9 +475,20 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1arrow_1set",
 	"_1gtk_1bin_1get_1child",
 	"_1gtk_1box_1set_1child_1packing",
+	"_1gtk_1box_1set_1spacing",
 	"_1gtk_1button_1clicked",
+	"_1gtk_1button_1get_1relief",
 	"_1gtk_1button_1new",
 	"_1gtk_1button_1set_1relief",
+	"_1gtk_1calendar_1display_1options",
+	"_1gtk_1calendar_1get_1date",
+	"_1gtk_1calendar_1new",
+	"_1gtk_1calendar_1select_1day",
+	"_1gtk_1calendar_1select_1month",
+	"_1gtk_1calendar_1set_1display_1options",
+	"_1gtk_1cell_1layout_1clear",
+	"_1gtk_1cell_1layout_1pack_1start",
+	"_1gtk_1cell_1layout_1set_1attributes",
 	"_1gtk_1cell_1renderer_1get_1size",
 	"_1gtk_1cell_1renderer_1pixbuf_1new",
 	"_1gtk_1cell_1renderer_1text_1new",
@@ -420,6 +506,16 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1color_1selection_1get_1current_1color",
 	"_1gtk_1color_1selection_1set_1current_1color",
 	"_1gtk_1color_1selection_1set_1has_1palette",
+	"_1gtk_1combo_1box_1entry_1new_1text",
+	"_1gtk_1combo_1box_1get_1active",
+	"_1gtk_1combo_1box_1get_1model",
+	"_1gtk_1combo_1box_1insert_1text",
+	"_1gtk_1combo_1box_1new_1text",
+	"_1gtk_1combo_1box_1popdown",
+	"_1gtk_1combo_1box_1popup",
+	"_1gtk_1combo_1box_1remove_1text",
+	"_1gtk_1combo_1box_1set_1active",
+	"_1gtk_1combo_1box_1set_1focus_1on_1click",
 	"_1gtk_1combo_1disable_1activate",
 	"_1gtk_1combo_1new",
 	"_1gtk_1combo_1set_1case_1sensitive",
@@ -440,6 +536,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1drag_1dest_1unset",
 	"_1gtk_1drag_1finish",
 	"_1gtk_1drag_1get_1data",
+	"_1gtk_1drag_1set_1icon_1pixbuf",
 	"_1gtk_1drawing_1area_1new",
 	"_1gtk_1editable_1copy_1clipboard",
 	"_1gtk_1editable_1cut_1clipboard",
@@ -456,6 +553,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1editable_1set_1position",
 	"_1gtk_1entry_1get_1invisible_1char",
 	"_1gtk_1entry_1get_1layout",
+	"_1gtk_1entry_1get_1layout_1offsets",
 	"_1gtk_1entry_1get_1max_1length",
 	"_1gtk_1entry_1get_1text",
 	"_1gtk_1entry_1get_1visibility",
@@ -467,18 +565,30 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1entry_1set_1max_1length",
 	"_1gtk_1entry_1set_1text",
 	"_1gtk_1entry_1set_1visibility",
+	"_1gtk_1entry_1text_1index_1to_1layout_1index",
+	"_1gtk_1enumerate_1printers",
 	"_1gtk_1events_1pending",
+	"_1gtk_1expander_1get_1expanded",
+	"_1gtk_1expander_1get_1label_1widget",
+	"_1gtk_1expander_1new",
+	"_1gtk_1expander_1set_1expanded",
+	"_1gtk_1expander_1set_1label",
+	"_1gtk_1expander_1set_1label_1widget",
 	"_1gtk_1file_1chooser_1add_1filter",
 	"_1gtk_1file_1chooser_1dialog_1new",
 	"_1gtk_1file_1chooser_1get_1current_1folder",
 	"_1gtk_1file_1chooser_1get_1filename",
 	"_1gtk_1file_1chooser_1get_1filenames",
+	"_1gtk_1file_1chooser_1get_1filter",
 	"_1gtk_1file_1chooser_1set_1current_1folder",
 	"_1gtk_1file_1chooser_1set_1current_1name",
+	"_1gtk_1file_1chooser_1set_1do_1overwrite_1confirmation",
 	"_1gtk_1file_1chooser_1set_1extra_1widget",
 	"_1gtk_1file_1chooser_1set_1filename",
+	"_1gtk_1file_1chooser_1set_1filter",
 	"_1gtk_1file_1chooser_1set_1select_1multiple",
 	"_1gtk_1file_1filter_1add_1pattern",
+	"_1gtk_1file_1filter_1get_1name",
 	"_1gtk_1file_1filter_1new",
 	"_1gtk_1file_1filter_1set_1name",
 	"_1gtk_1file_1selection_1get_1filename",
@@ -512,6 +622,9 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1hseparator_1new",
 	"_1gtk_1icon_1factory_1lookup_1default",
 	"_1gtk_1icon_1set_1render_1icon",
+	"_1gtk_1icon_1source_1free",
+	"_1gtk_1icon_1source_1new",
+	"_1gtk_1icon_1source_1set_1pixbuf",
 	"_1gtk_1im_1context_1filter_1keypress",
 	"_1gtk_1im_1context_1focus_1in",
 	"_1gtk_1im_1context_1focus_1out",
@@ -530,12 +643,14 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1image_1set_1from_1pixbuf",
 	"_1gtk_1image_1set_1from_1pixmap",
 	"_1gtk_1init_1check",
+	"_1gtk_1label_1get_1layout",
 	"_1gtk_1label_1get_1mnemonic_1keyval",
 	"_1gtk_1label_1new",
 	"_1gtk_1label_1new_1with_1mnemonic",
 	"_1gtk_1label_1set_1attributes",
 	"_1gtk_1label_1set_1justify",
 	"_1gtk_1label_1set_1line_1wrap",
+	"_1gtk_1label_1set_1line_1wrap_1mode",
 	"_1gtk_1label_1set_1text__II",
 	"_1gtk_1label_1set_1text__I_3B",
 	"_1gtk_1label_1set_1text_1with_1mnemonic",
@@ -562,6 +677,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1main_1iteration",
 	"_1gtk_1major_1version",
 	"_1gtk_1menu_1bar_1new",
+	"_1gtk_1menu_1item_1get_1submenu",
 	"_1gtk_1menu_1item_1remove_1submenu",
 	"_1gtk_1menu_1item_1set_1submenu",
 	"_1gtk_1menu_1new",
@@ -570,6 +686,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1menu_1shell_1deactivate",
 	"_1gtk_1menu_1shell_1insert",
 	"_1gtk_1menu_1shell_1select_1item",
+	"_1gtk_1menu_1shell_1set_1take_1focus",
 	"_1gtk_1message_1dialog_1new",
 	"_1gtk_1micro_1version",
 	"_1gtk_1minor_1version",
@@ -578,16 +695,99 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1notebook_1get_1scrollable",
 	"_1gtk_1notebook_1insert_1page",
 	"_1gtk_1notebook_1new",
+	"_1gtk_1notebook_1next_1page",
+	"_1gtk_1notebook_1prev_1page",
 	"_1gtk_1notebook_1remove_1page",
 	"_1gtk_1notebook_1set_1current_1page",
 	"_1gtk_1notebook_1set_1scrollable",
 	"_1gtk_1notebook_1set_1show_1tabs",
 	"_1gtk_1notebook_1set_1tab_1pos",
 	"_1gtk_1object_1sink",
+	"_1gtk_1page_1setup_1get_1bottom_1margin",
+	"_1gtk_1page_1setup_1get_1left_1margin",
+	"_1gtk_1page_1setup_1get_1orientation",
+	"_1gtk_1page_1setup_1get_1page_1height",
+	"_1gtk_1page_1setup_1get_1page_1width",
+	"_1gtk_1page_1setup_1get_1paper_1height",
+	"_1gtk_1page_1setup_1get_1paper_1size",
+	"_1gtk_1page_1setup_1get_1paper_1width",
+	"_1gtk_1page_1setup_1get_1right_1margin",
+	"_1gtk_1page_1setup_1get_1top_1margin",
+	"_1gtk_1page_1setup_1new",
+	"_1gtk_1page_1setup_1set_1bottom_1margin",
+	"_1gtk_1page_1setup_1set_1left_1margin",
+	"_1gtk_1page_1setup_1set_1orientation",
+	"_1gtk_1page_1setup_1set_1paper_1size",
+	"_1gtk_1page_1setup_1set_1right_1margin",
+	"_1gtk_1page_1setup_1set_1top_1margin",
+	"_1gtk_1paint_1arrow",
+	"_1gtk_1paint_1box",
+	"_1gtk_1paint_1box_1gap",
+	"_1gtk_1paint_1check",
+	"_1gtk_1paint_1expander",
+	"_1gtk_1paint_1extension",
+	"_1gtk_1paint_1flat_1box",
 	"_1gtk_1paint_1focus",
 	"_1gtk_1paint_1handle",
+	"_1gtk_1paint_1hline",
+	"_1gtk_1paint_1layout",
+	"_1gtk_1paint_1option",
+	"_1gtk_1paint_1shadow",
+	"_1gtk_1paint_1shadow_1gap",
+	"_1gtk_1paint_1slider",
+	"_1gtk_1paint_1tab",
+	"_1gtk_1paint_1vline",
+	"_1gtk_1paper_1size_1free",
+	"_1gtk_1paper_1size_1get_1display_1name",
+	"_1gtk_1paper_1size_1get_1height",
+	"_1gtk_1paper_1size_1get_1name",
+	"_1gtk_1paper_1size_1get_1ppd_1name",
+	"_1gtk_1paper_1size_1get_1width",
+	"_1gtk_1paper_1size_1is_1custom",
+	"_1gtk_1paper_1size_1new",
+	"_1gtk_1paper_1size_1new_1custom",
+	"_1gtk_1paper_1size_1new_1from_1ppd",
 	"_1gtk_1plug_1get_1id",
 	"_1gtk_1plug_1new",
+	"_1gtk_1print_1job_1get_1printer",
+	"_1gtk_1print_1job_1get_1settings",
+	"_1gtk_1print_1job_1get_1status",
+	"_1gtk_1print_1job_1get_1surface",
+	"_1gtk_1print_1job_1get_1title",
+	"_1gtk_1print_1job_1new",
+	"_1gtk_1print_1job_1send",
+	"_1gtk_1print_1job_1set_1source_1file",
+	"_1gtk_1print_1settings_1foreach",
+	"_1gtk_1print_1settings_1get",
+	"_1gtk_1print_1settings_1get_1collate",
+	"_1gtk_1print_1settings_1get_1n_1copies",
+	"_1gtk_1print_1settings_1get_1orientation",
+	"_1gtk_1print_1settings_1get_1page_1ranges",
+	"_1gtk_1print_1settings_1get_1paper_1height",
+	"_1gtk_1print_1settings_1get_1paper_1width",
+	"_1gtk_1print_1settings_1get_1print_1pages",
+	"_1gtk_1print_1settings_1get_1printer",
+	"_1gtk_1print_1settings_1get_1resolution",
+	"_1gtk_1print_1settings_1new",
+	"_1gtk_1print_1settings_1set",
+	"_1gtk_1print_1settings_1set_1collate",
+	"_1gtk_1print_1settings_1set_1n_1copies",
+	"_1gtk_1print_1settings_1set_1orientation",
+	"_1gtk_1print_1settings_1set_1page_1ranges",
+	"_1gtk_1print_1settings_1set_1print_1pages",
+	"_1gtk_1print_1settings_1set_1printer",
+	"_1gtk_1print_1unix_1dialog_1get_1current_1page",
+	"_1gtk_1print_1unix_1dialog_1get_1page_1setup",
+	"_1gtk_1print_1unix_1dialog_1get_1selected_1printer",
+	"_1gtk_1print_1unix_1dialog_1get_1settings",
+	"_1gtk_1print_1unix_1dialog_1new",
+	"_1gtk_1print_1unix_1dialog_1set_1current_1page",
+	"_1gtk_1print_1unix_1dialog_1set_1manual_1capabilities",
+	"_1gtk_1print_1unix_1dialog_1set_1page_1setup",
+	"_1gtk_1print_1unix_1dialog_1set_1settings",
+	"_1gtk_1printer_1get_1backend",
+	"_1gtk_1printer_1get_1name",
+	"_1gtk_1printer_1is_1default",
 	"_1gtk_1progress_1bar_1new",
 	"_1gtk_1progress_1bar_1pulse",
 	"_1gtk_1progress_1bar_1set_1fraction",
@@ -608,8 +808,11 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1rc_1style_1set_1bg",
 	"_1gtk_1rc_1style_1set_1bg_1pixmap_1name",
 	"_1gtk_1rc_1style_1set_1color_1flags",
+	"_1gtk_1rc_1style_1set_1fg",
+	"_1gtk_1rc_1style_1set_1text",
 	"_1gtk_1scale_1set_1digits",
 	"_1gtk_1scale_1set_1draw_1value",
+	"_1gtk_1scrolled_1window_1add_1with_1viewport",
 	"_1gtk_1scrolled_1window_1get_1hadjustment",
 	"_1gtk_1scrolled_1window_1get_1policy",
 	"_1gtk_1scrolled_1window_1get_1shadow_1type",
@@ -633,16 +836,27 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1spin_1button_1set_1range",
 	"_1gtk_1spin_1button_1set_1value",
 	"_1gtk_1spin_1button_1set_1wrap",
+	"_1gtk_1spin_1button_1update",
 	"_1gtk_1style_1get_1base",
 	"_1gtk_1style_1get_1bg",
+	"_1gtk_1style_1get_1bg_1gc",
 	"_1gtk_1style_1get_1black",
+	"_1gtk_1style_1get_1black_1gc",
 	"_1gtk_1style_1get_1dark",
+	"_1gtk_1style_1get_1dark_1gc",
 	"_1gtk_1style_1get_1fg",
+	"_1gtk_1style_1get_1fg_1gc",
 	"_1gtk_1style_1get_1font_1desc",
 	"_1gtk_1style_1get_1light",
+	"_1gtk_1style_1get_1light_1gc",
+	"_1gtk_1style_1get_1mid_1gc",
 	"_1gtk_1style_1get_1text",
+	"_1gtk_1style_1get_1text_1aa_1gc",
+	"_1gtk_1style_1get_1text_1gc",
+	"_1gtk_1style_1get_1white_1gc",
 	"_1gtk_1style_1get_1xthickness",
 	"_1gtk_1style_1get_1ythickness",
+	"_1gtk_1style_1render_1icon",
 	"_1gtk_1target_1list_1new",
 	"_1gtk_1target_1list_1unref",
 	"_1gtk_1text_1buffer_1copy_1clipboard",
@@ -686,14 +900,19 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1timeout_1add",
 	"_1gtk_1timeout_1remove",
 	"_1gtk_1toggle_1button_1get_1active",
+	"_1gtk_1toggle_1button_1get_1inconsistent",
 	"_1gtk_1toggle_1button_1new",
 	"_1gtk_1toggle_1button_1set_1active",
+	"_1gtk_1toggle_1button_1set_1inconsistent",
 	"_1gtk_1toggle_1button_1set_1mode",
 	"_1gtk_1toolbar_1insert_1widget",
 	"_1gtk_1toolbar_1new",
 	"_1gtk_1toolbar_1set_1orientation",
+	"_1gtk_1tooltip_1trigger_1tooltip_1query",
+	"_1gtk_1tooltips_1data_1get",
 	"_1gtk_1tooltips_1disable",
 	"_1gtk_1tooltips_1enable",
+	"_1gtk_1tooltips_1force_1window",
 	"_1gtk_1tooltips_1new",
 	"_1gtk_1tooltips_1set_1tip",
 	"_1gtk_1tree_1model_1get__III_3II",
@@ -720,6 +939,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1tree_1path_1next",
 	"_1gtk_1tree_1path_1prev",
 	"_1gtk_1tree_1path_1up",
+	"_1gtk_1tree_1selection_1count_1selected_1rows",
 	"_1gtk_1tree_1selection_1get_1selected",
 	"_1gtk_1tree_1selection_1get_1selected_1rows",
 	"_1gtk_1tree_1selection_1path_1is_1selected",
@@ -749,6 +969,9 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1tree_1view_1column_1get_1fixed_1width",
 	"_1gtk_1tree_1view_1column_1get_1reorderable",
 	"_1gtk_1tree_1view_1column_1get_1resizable",
+	"_1gtk_1tree_1view_1column_1get_1sizing",
+	"_1gtk_1tree_1view_1column_1get_1sort_1indicator",
+	"_1gtk_1tree_1view_1column_1get_1sort_1order",
 	"_1gtk_1tree_1view_1column_1get_1spacing",
 	"_1gtk_1tree_1view_1column_1get_1visible",
 	"_1gtk_1tree_1view_1column_1get_1width",
@@ -759,13 +982,18 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1tree_1view_1column_1set_1cell_1data_1func",
 	"_1gtk_1tree_1view_1column_1set_1clickable",
 	"_1gtk_1tree_1view_1column_1set_1fixed_1width",
+	"_1gtk_1tree_1view_1column_1set_1min_1width",
 	"_1gtk_1tree_1view_1column_1set_1reorderable",
 	"_1gtk_1tree_1view_1column_1set_1resizable",
 	"_1gtk_1tree_1view_1column_1set_1sizing",
+	"_1gtk_1tree_1view_1column_1set_1sort_1indicator",
+	"_1gtk_1tree_1view_1column_1set_1sort_1order",
 	"_1gtk_1tree_1view_1column_1set_1title",
 	"_1gtk_1tree_1view_1column_1set_1visible",
 	"_1gtk_1tree_1view_1column_1set_1widget",
+	"_1gtk_1tree_1view_1create_1row_1drag_1icon",
 	"_1gtk_1tree_1view_1expand_1row",
+	"_1gtk_1tree_1view_1get_1background_1area",
 	"_1gtk_1tree_1view_1get_1bin_1window",
 	"_1gtk_1tree_1view_1get_1cell_1area",
 	"_1gtk_1tree_1view_1get_1column",
@@ -786,6 +1014,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1tree_1view_1scroll_1to_1point",
 	"_1gtk_1tree_1view_1set_1cursor",
 	"_1gtk_1tree_1view_1set_1drag_1dest_1row",
+	"_1gtk_1tree_1view_1set_1enable_1search",
 	"_1gtk_1tree_1view_1set_1headers_1visible",
 	"_1gtk_1tree_1view_1set_1model",
 	"_1gtk_1tree_1view_1set_1rules_1hint",
@@ -804,6 +1033,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1widget_1create_1pango_1layout__I_3B",
 	"_1gtk_1widget_1destroy",
 	"_1gtk_1widget_1event",
+	"_1gtk_1widget_1get_1accessible",
 	"_1gtk_1widget_1get_1child_1visible",
 	"_1gtk_1widget_1get_1default_1direction",
 	"_1gtk_1widget_1get_1default_1style",
@@ -817,6 +1047,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1widget_1get_1toplevel",
 	"_1gtk_1widget_1grab_1focus",
 	"_1gtk_1widget_1hide",
+	"_1gtk_1widget_1is_1composited",
 	"_1gtk_1widget_1is_1focus",
 	"_1gtk_1widget_1map",
 	"_1gtk_1widget_1mnemonic_1activate",
@@ -826,17 +1057,23 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1widget_1modify_1font",
 	"_1gtk_1widget_1modify_1style",
 	"_1gtk_1widget_1modify_1text",
+	"_1gtk_1widget_1queue_1resize",
 	"_1gtk_1widget_1realize",
 	"_1gtk_1widget_1remove_1accelerator",
 	"_1gtk_1widget_1reparent",
+	"_1gtk_1widget_1send_1expose",
+	"_1gtk_1widget_1set_1app_1paintable",
 	"_1gtk_1widget_1set_1default_1direction",
 	"_1gtk_1widget_1set_1direction",
 	"_1gtk_1widget_1set_1double_1buffered",
 	"_1gtk_1widget_1set_1name",
+	"_1gtk_1widget_1set_1parent_1window",
 	"_1gtk_1widget_1set_1redraw_1on_1allocate",
 	"_1gtk_1widget_1set_1sensitive",
 	"_1gtk_1widget_1set_1size_1request",
 	"_1gtk_1widget_1set_1state",
+	"_1gtk_1widget_1set_1style",
+	"_1gtk_1widget_1set_1tooltip_1text",
 	"_1gtk_1widget_1shape_1combine_1mask",
 	"_1gtk_1widget_1show",
 	"_1gtk_1widget_1show_1now",
@@ -844,16 +1081,25 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1widget_1size_1request",
 	"_1gtk_1widget_1style_1get__I_3B_3II",
 	"_1gtk_1widget_1style_1get__I_3B_3JI",
+	"_1gtk_1widget_1translate_1coordinates",
 	"_1gtk_1widget_1unrealize",
 	"_1gtk_1window_1activate_1default",
 	"_1gtk_1window_1add_1accel_1group",
 	"_1gtk_1window_1deiconify",
+	"_1gtk_1window_1fullscreen",
 	"_1gtk_1window_1get_1focus",
+	"_1gtk_1window_1get_1group",
 	"_1gtk_1window_1get_1icon_1list",
 	"_1gtk_1window_1get_1mnemonic_1modifier",
+	"_1gtk_1window_1get_1modal",
+	"_1gtk_1window_1get_1opacity",
 	"_1gtk_1window_1get_1position",
 	"_1gtk_1window_1get_1size",
+	"_1gtk_1window_1group_1add_1window",
+	"_1gtk_1window_1group_1new",
+	"_1gtk_1window_1group_1remove_1window",
 	"_1gtk_1window_1iconify",
+	"_1gtk_1window_1list_1toplevels",
 	"_1gtk_1window_1maximize",
 	"_1gtk_1window_1move",
 	"_1gtk_1window_1new",
@@ -864,23 +1110,45 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1window_1set_1destroy_1with_1parent",
 	"_1gtk_1window_1set_1geometry_1hints",
 	"_1gtk_1window_1set_1icon_1list",
+	"_1gtk_1window_1set_1keep_1below",
 	"_1gtk_1window_1set_1modal",
+	"_1gtk_1window_1set_1opacity",
 	"_1gtk_1window_1set_1resizable",
+	"_1gtk_1window_1set_1skip_1taskbar_1hint",
 	"_1gtk_1window_1set_1title",
 	"_1gtk_1window_1set_1transient_1for",
 	"_1gtk_1window_1set_1type_1hint",
+	"_1gtk_1window_1unfullscreen",
 	"_1gtk_1window_1unmaximize",
 	"_1pango_1attr_1background_1new",
 	"_1pango_1attr_1font_1desc_1new",
 	"_1pango_1attr_1foreground_1new",
+	"_1pango_1attr_1iterator_1destroy",
+	"_1pango_1attr_1iterator_1get",
+	"_1pango_1attr_1iterator_1get_1attrs",
+	"_1pango_1attr_1iterator_1next",
+	"_1pango_1attr_1iterator_1range",
 	"_1pango_1attr_1list_1change",
+	"_1pango_1attr_1list_1get_1iterator",
 	"_1pango_1attr_1list_1insert",
 	"_1pango_1attr_1list_1new",
 	"_1pango_1attr_1list_1unref",
+	"_1pango_1attr_1rise_1new",
 	"_1pango_1attr_1shape_1new",
+	"_1pango_1attr_1strikethrough_1color_1new",
 	"_1pango_1attr_1strikethrough_1new",
+	"_1pango_1attr_1underline_1color_1new",
 	"_1pango_1attr_1underline_1new",
 	"_1pango_1attr_1weight_1new",
+	"_1pango_1cairo_1context_1get_1font_1options",
+	"_1pango_1cairo_1context_1set_1font_1options",
+	"_1pango_1cairo_1create_1layout",
+	"_1pango_1cairo_1font_1map_1create_1context",
+	"_1pango_1cairo_1font_1map_1get_1default",
+	"_1pango_1cairo_1font_1map_1new",
+	"_1pango_1cairo_1font_1map_1set_1resolution",
+	"_1pango_1cairo_1layout_1path",
+	"_1pango_1cairo_1show_1layout",
 	"_1pango_1context_1get_1base_1dir",
 	"_1pango_1context_1get_1language",
 	"_1pango_1context_1get_1metrics",
@@ -908,12 +1176,19 @@ char * OS_nativeFunctionNames[] = {
 	"_1pango_1font_1metrics_1get_1approximate_1char_1width",
 	"_1pango_1font_1metrics_1get_1ascent",
 	"_1pango_1font_1metrics_1get_1descent",
+	"_1pango_1font_1metrics_1get_1strikethrough_1position",
+	"_1pango_1font_1metrics_1get_1strikethrough_1thickness",
+	"_1pango_1font_1metrics_1get_1underline_1position",
+	"_1pango_1font_1metrics_1get_1underline_1thickness",
 	"_1pango_1font_1metrics_1unref",
 	"_1pango_1language_1from_1string",
 	"_1pango_1layout_1context_1changed",
 	"_1pango_1layout_1get_1alignment",
 	"_1pango_1layout_1get_1attributes",
+	"_1pango_1layout_1get_1context",
+	"_1pango_1layout_1get_1indent",
 	"_1pango_1layout_1get_1iter",
+	"_1pango_1layout_1get_1justify",
 	"_1pango_1layout_1get_1line",
 	"_1pango_1layout_1get_1line_1count",
 	"_1pango_1layout_1get_1log_1attrs",
@@ -930,11 +1205,15 @@ char * OS_nativeFunctionNames[] = {
 	"_1pango_1layout_1iter_1next_1line",
 	"_1pango_1layout_1iter_1next_1run",
 	"_1pango_1layout_1line_1get_1extents",
+	"_1pango_1layout_1line_1get_1resolved_1dir",
 	"_1pango_1layout_1line_1x_1to_1index",
 	"_1pango_1layout_1new",
 	"_1pango_1layout_1set_1alignment",
 	"_1pango_1layout_1set_1attributes",
+	"_1pango_1layout_1set_1auto_1dir",
 	"_1pango_1layout_1set_1font_1description",
+	"_1pango_1layout_1set_1indent",
+	"_1pango_1layout_1set_1justify",
 	"_1pango_1layout_1set_1single_1paragraph_1mode",
 	"_1pango_1layout_1set_1spacing",
 	"_1pango_1layout_1set_1tabs",
@@ -948,14 +1227,15 @@ char * OS_nativeFunctionNames[] = {
 	"_1pango_1tab_1array_1new",
 	"_1pango_1tab_1array_1set_1tab",
 	"g_1main_1context_1wakeup",
-	"getenv",
 	"localeconv_1decimal_1point",
-	"memmove__III",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GInterfaceInfo_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GObjectClass_2",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GTypeInfo_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GdkEventButton_2I",
+	"memmove__ILorg_eclipse_swt_internal_gtk_GdkEventExpose_2I",
+	"memmove__ILorg_eclipse_swt_internal_gtk_GdkEventMotion_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GtkAdjustment_2",
+	"memmove__ILorg_eclipse_swt_internal_gtk_GtkCellRendererClass_2",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GtkFixed_2",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GtkTargetEntry_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_GtkWidgetClass_2",
@@ -965,15 +1245,11 @@ char * OS_nativeFunctionNames[] = {
 	"memmove__ILorg_eclipse_swt_internal_gtk_XCrossingEvent_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_XExposeEvent_2I",
 	"memmove__ILorg_eclipse_swt_internal_gtk_XFocusChangeEvent_2I",
-	"memmove__I_3BI",
-	"memmove__I_3CI",
-	"memmove__I_3DI",
-	"memmove__I_3II",
-	"memmove__I_3JI",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GObjectClass_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GTypeQuery_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkColor_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkDragContext_2II",
+	"memmove__Lorg_eclipse_swt_internal_gtk_GdkEventAny_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkEventButton_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkEventCrossing_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkEventExpose_2II",
@@ -989,6 +1265,7 @@ char * OS_nativeFunctionNames[] = {
 	"memmove__Lorg_eclipse_swt_internal_gtk_GdkVisual_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkAdjustment_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkBorder_2II",
+	"memmove__Lorg_eclipse_swt_internal_gtk_GtkCellRendererClass_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkColorSelectionDialog_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkCombo_2I",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkFileSelection_2I",
@@ -996,6 +1273,8 @@ char * OS_nativeFunctionNames[] = {
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkSelectionData_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkTargetPair_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_GtkWidgetClass_2I",
+	"memmove__Lorg_eclipse_swt_internal_gtk_PangoAttrColor_2II",
+	"memmove__Lorg_eclipse_swt_internal_gtk_PangoAttrInt_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_PangoAttribute_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_PangoItem_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_PangoLayoutLine_2II",
@@ -1006,13 +1285,7 @@ char * OS_nativeFunctionNames[] = {
 	"memmove__Lorg_eclipse_swt_internal_gtk_XExposeEvent_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_XFocusChangeEvent_2II",
 	"memmove__Lorg_eclipse_swt_internal_gtk_XVisibilityEvent_2II",
-	"memmove___3BII",
-	"memmove___3CII",
-	"memmove___3III",
-	"memmove___3I_3BI",
-	"memmove___3JII",
-	"memset",
-	"strlen",
+	"realpath",
 };
 
 #define STATS_NATIVE(func) Java_org_eclipse_swt_tools_internal_NativeStats_##func
